@@ -4,15 +4,20 @@
 from .base import *
 from .helper import *
 from .logic.bitwise import *
+from .logic.clock import *
 from .logic.storage import *
 from .logic.arithmetic import *
 from .logic.arithmetic_fp import *
+from .logic.arithmetic_fxp import *
 from .logic.relational import *
 from .logic.simulation import *
 from .schematic import *
 from .rtl_generation import *
+from .py4hw_generation import *
+from .transpilation import *
 
 import py4hw.gui
 import py4hw.debug
 
-from nbwavedrom import draw as draw_waveform
\ No newline at end of file
+
+from nbwavedrom import draw as draw_waveform
