@@ -21,11 +21,14 @@ class Logic:
 
         self.inPorts = []   # in ports are a sorted list
         self.outPorts = []  # out ports are a sorted list
+        self.inOutPorts = [] # in/out ports
         self.sources = []   # list of InterfaceSource objects
         self.sinks = []     # List of InterfaceSink objects
     
         self.children = {}          # children are keyed by name
         self.clockDriver = None     # every circuit has a clock driver, if None it is inherited from parent
+                                    # the clock driver in place for an object can be obtained by the function
+                                    # getObjectClockDriver(obj)
         
         self._wires = {}        # dictionary of wires created by the object
         
@@ -56,7 +59,46 @@ class Logic:
         port = OutPort(self, name, wire)
         self.outPorts.append(port)
         return wire;
+    
+    def addInOut(self, name, wire):
+        port = InOutPort(self, name, wire)
+        self.inOutPorts.append(port)
+        return wire;
 
+    def addParameter(self, name, value):
+        if not(hasattr(self, 'parameters')):
+            self.parameters = {}
+            
+        self.parameters[name] = value
+        
+    def getParameterValue(self, name):
+        value = self.parameters[name]
+        
+        if (isinstance(value, Parameter)):
+            value = value.obj.parameters[value.name]
+        return value
+    
+    def getParameterInstantiationValue(self, name):
+        # returns the final name associated with this parameter, used in rtl
+        # generation.
+        # If the parent instantiates the parameter with a value, we return it
+        # If the parent instantiates the parameter with a parent parameter, we return it
+        value = self.parameters[name]
+
+        if (isinstance(value, Parameter)):
+            return value.name
+        else:
+            return value            
+    
+    def getParameter(self, name):
+        return Parameter(self, name)        
+    
+    def getParameterNames(self):
+        if not(hasattr(self, 'parameters')):
+            return None
+        
+        return self.parameters.keys()
+    
     def addInterfaceSource(self, name:str, interface):
         """
         Adds the source ports of the interface to the 
@@ -72,16 +114,19 @@ class Logic:
 
         """
         ports = []
+        if (len(name) > 0):
+            name = name + '_'
+            
         for obj in interface.sourceToSink:
             wire = obj[1]
-            portname = name + "_" +obj[0]
+            portname = name + obj[0]
             port = OutPort(self, portname, wire)
             self.outPorts.append(port)
             ports.append(port)
         
         for obj in interface.sinkToSource:
             wire = obj[1]
-            portname = name + "_" +obj[0]
+            portname = name + obj[0]
             port = InPort(self, portname, wire)
             self.inPorts.append(port)
             ports.append(port)
@@ -105,15 +150,18 @@ class Logic:
 
         """
         ports = []
+        if (len(name) > 0):
+            name = name + '_'
+            
         for obj in interface.sourceToSink:
             wire = obj[1]
-            portname = name + "_" +obj[0]
+            portname = name + obj[0]
             port = InPort(self, portname, wire)
             self.inPorts.append(port)
             ports.append(port)
         for obj in interface.sinkToSource:
             wire = obj[1]
-            portname = name + "_" +obj[0]
+            portname = name + obj[0]
             port = OutPort(self, portname, wire)
             self.outPorts.append(port)
             ports.append(port)
@@ -129,6 +177,9 @@ class Logic:
             
         self._wires[wire.name] = wire
         
+    def bidir_wire(self, name, width=1):
+        return BidirWire(self, name, width)
+    
     def wire(self, name, width=1):
         return Wire(self, name, width);
     
@@ -172,6 +223,8 @@ class Logic:
     def isClockable(self):
         return has_method(self, "clock")
     
+    def isRunnable(self):
+        return has_method(self, "run")
     
     # def getHierarchy(self):
     #     str = ''
@@ -256,9 +309,15 @@ class Logic:
         else:
             return objFound;
         
+class Parameter:
+    def __init__(self, obj, name):
+        self.obj = obj
+        self.name = name
+        
 class Wire:
     """
-    Wires in py4hw connect one source with one (or more) sinks
+    Wires in py4hw connect one source with one (or more) sinks.
+    For many-to-many connections use BidirWire
     """
     
     # this is the list of prepared wires, 
@@ -288,6 +347,9 @@ class Wire:
         self.value = val & mask
 
     def prepare(self, val:int):
+        if (self in Wire.prepared):
+            print('WARNING! wire {} already prepared'.format(self.getFullPath()))
+            
         mask = (1<<self.width) -1
         self.next = val & mask
         Wire.prepared.append(self)
@@ -304,6 +366,129 @@ class Wire:
 
         self.source = source
         
+    def addSource(self, source):
+        self.setSource(source)
+        
+    def getSource(self):
+        """
+        Returns the OutPort that drives this wire
+
+        Returns
+        -------
+        TYPE
+            DESCRIPTION.
+
+        """
+        return self.source
+    
+    def addSink(self, sink):
+        """
+        Adds a sink to the wire
+
+        Parameters
+        ----------
+        sink : TYPE
+            DESCRIPTION.
+
+        Returns
+        -------
+        None.
+
+        """
+        self.sinks.append(sink)
+        
+    def getSinks(self):
+        """
+        Gets all the sinks from a wire
+
+        Returns
+        -------
+        TYPE
+            DESCRIPTION.
+
+        """
+        return self.sinks;
+    
+
+    def settleAll():
+        """
+        Settles all pending wires that were changed by clock methods
+
+        Returns
+        -------
+        None.
+
+        """
+        
+        for w in Wire.prepared:
+            w.settle()
+            
+        # empty list
+        Wire.prepared = []
+    
+    def rename(self, newname):
+        del self.parent._wires[self.name]
+        self.name = newname
+        self.parent.appendWire(self)
+        
+    def reparent(self, newparent):
+        del self.parent._wires[self.name]
+        self.parent = newparent
+        newparent.appendWire(self)
+
+    def reparentAndRename(self, newparent, newname):
+        del self.parent._wires[self.name]
+        self.name = newname
+        self.parent = newparent
+        newparent.appendWire(self)
+
+class BidirWire(Wire):
+    """
+    Wires in py4hw connect one source with one (or more) sinks.
+    For many-to-many connections use BidirWire
+    """
+    
+    # this is the list of prepared wires, 
+    # @todo what for ??
+    prepared = []
+    
+    def __init__(self, parent, name : str, width: int = 1 ):
+        # the following should not be necessary if python checks types
+        assert(isinstance(name, str))
+        assert(isinstance(width, int))
+        
+        self.parent = parent
+        self.name = name
+        self.width = width
+        self.value = 0 # should be None      # reset state
+        self.sinks = []
+        self.sources = []
+        parent.appendWire(self)
+        
+    def getFullPath(self)->str:
+        return self.parent.getFullPath() + '[{}]'.format(self.name)
+    
+    def getWidth(self) -> int:
+        return self.width
+    
+    def put(self, val:int):
+        mask = (1<<self.width) -1
+        self.value = val & mask
+
+    def prepare(self, val:int):
+        mask = (1<<self.width) -1
+        self.next = val & mask
+        Wire.prepared.append(self)
+        
+    def settle(self):
+        self.value = self.next
+        
+    def get(self) -> int:
+        return self.value
+    
+    def addSource(self, source):        
+        self.sources.append(source)
+        
         
     def getSource(self):
         """
@@ -376,6 +561,25 @@ class Wire:
         self.name = newname
         self.parent = newparent
         newparent.appendWire(self)
+        
+class FakeWire():
+    # By now we use FakeWire to refer to a specific subwire of a bidirectional
+    # port.
+    def __init__(self, name, w=1):
+        self.name = name
+        self.w = w
+
+    def addSource(self, source):
+        pass
+    
+    def addSink(self, sink):
+        pass
+    
+    def getWidth(self):
+        return self.w
+    
+    def getFullPath(self):
+        return self.name
 
 class InPort:
     """
@@ -417,7 +621,7 @@ class OutPort:
     """
     An output port
     """
-    def __init__(self, parent:Logic, name:str, wire:Wire):
+    def __init__(self, parent:Logic, name:str, wire):
         """
         Creates an out port to the cell.
         The cell will be registered as source of the wire only if it
@@ -443,7 +647,44 @@ class OutPort:
         self.wire = wire
         
         if (parent.isPrimitive()):
-            wire.setSource(self)
+            wire.addSource(self)
+
+    def getFullPath(self):
+        return self.parent.getFullPath() + '[{}]'.format(self.name)
+
+class InOutPort:
+    """
+    An input/output port
+    """
+    def __init__(self, parent:Logic, name:str, wire:Wire):
+        """
+        Creates an in/out port to the cell.
+        The cell will be registered as source of the wire only if it
+        is a leaf (a primitive) in the hierarchy 
+
+        Parameters
+        ----------
+        parent : Logic
+            Parent Cell of the port
+        name : str
+            name of the port.
+        wire : Wire
+            Wire associated with the port
+
+        Returns
+        -------
+        None.
+
+        """
+        #print('out port')
+        self.name = name
+        self.parent = parent
+        self.wire = wire
+        
+        if (parent.isPrimitive()):
+            wire.addSource(self)
+            wire.addSink(self)
+
 
     def getFullPath(self):
         return self.parent.getFullPath() + '[{}]'.format(self.name)
@@ -464,13 +705,16 @@ class HWSystem(Logic):
     """
     A Hardware system is the top level entity of all designs 
     """
-    def __init__(self, clock_driver=None):
+    def __init__(self, clock_driver=None, name=None):
         super().__init__(None, "HWSystem")
         self.simulator = None
         if (clock_driver is None):
-            clock_driver = ClockDriver('clk50', 50E6, 0) # we create a 50MHz clock driver by default
+            clock_driver = ClockDriver('clk', 50E6, 0, wire=self.wire('clk')) # we create a 50MHz clock driver by default
+        if not(name is None):
+            self.name = name
         self.clockDriver = clock_driver
         
+        
     def getSimulator(self):
         """
         Returns the singleton simulator instance
@@ -496,7 +740,7 @@ class ClockDriver():
     A clock driver
     """
     
-    def __init__(self, name:str, freq=50E6, phaseOffset=0, base=None, enable=None):
+    def __init__(self, name:str, freq=50E6, phaseOffset=0, base=None, enable=None, wire=None):
         """
         Creates a clock driver
 
@@ -513,6 +757,8 @@ class ClockDriver():
         enable : Wire, optional
             Wire controling the gating of the clock. When enable is 0, the clock
             will be gated
+        wire : Wire, optional
+            Wire that contains the clock
 
         Returns
         -------
@@ -530,7 +776,7 @@ class ClockDriver():
             self.phaseOffset = base.phaseOffset
             
         self.enable = enable
-        
+        self.wire = wire
 
 def has_method(o, name):
     return callable(getattr(o, name, None))
@@ -563,9 +809,46 @@ class Interface:
         None.
 
         """
+        assert(isinstance(width, int))
         w = self.parent.wire(self.name + "_" + name, width)
         self.sourceToSink.append([name, w])
         return w;
+    
+    def removeSourceToSink(self, name:str):
+        if len(self.sourceToSink) == 0:
+            raise Exception('No source-to-sink elements in this interface')
+            
+        for i in range(len(self.sourceToSink)):
+            p = self.sourceToSink[i]
+            if (p[0] == name):
+                self.sourceToSink.pop(i)
+                
+                if (hasattr(self, name)):
+                    delattr(self, name)
+                return
+            
+        raise Exception('SourceToSink {} not found'.format(name))
+        
+    def getSourceToSink(self, name):
+        if len(self.sourceToSink) == 0:
+            raise Exception('No source-to-sink elements in this interface')
+            
+        for p in self.sourceToSink:
+            if (p[0] == name):
+                return p[1]
+            
+        raise Exception('SourceToSink {} not found'.format(name))
+    
+    def getSinkToSource(self, name):
+        # returns a wire with that name
+        if len(self.sinkToSource) == 0:
+            raise Exception('No sink-to-source elements in this interface')
+            
+        for p in self.sinkToSource:
+            if (p[0] == name):
+                return p[1]
+                
+        raise Exception('SinkToSource {} not found'.format(name))
         
     def addSinkToSource(self, name:str, width:int):
         """
@@ -586,6 +869,34 @@ class Interface:
         w = self.parent.wire(self.name + "_" + name, width)
         self.sinkToSource.append([name, w])
         return w;
+    
+    def removeSinkToSource(self, name:str):
+        if len(self.sinkToSource) == 0:
+            raise Exception('No sink-to-source elements in this interface')
+            
+        for i in range(len(self.sinkToSource)):
+            p = self.sinkToSource[i]
+            if (p[0] == name):
+                self.sinkToSource.pop(i)
+                
+                if (hasattr(self, name)):
+                    delattr(self, name)
+                    
+                return
+            
+        raise Exception('SinkToSource {} not found'.format(name))
+        
+    def addSourceToSinkRef(self, ref, name:str):
+        # Adds a reference to another interface, this is used in subinterfaces
+        w = ref.getSourceToSink(name)
+        self.sourceToSink.append([name, w])
+        return w
+        
+    def addSinkToSourceRef(self, ref, name:str):
+        # Adds a reference to another interface, this is used in subinterfaces
+        w = ref.getSinkToSource(name)
+        self.sinkToSource.append([name, w])
+        return w
         
 
 def disconnectWireFromLogicObject(w:Wire, obj:Logic):
@@ -647,4 +958,14 @@ def getObjectClockDriver(obj:Logic) -> ClockDriver:
     if (obj.parent == None):
         raise Exception('No clock driver at top level')
     else:
-        return getObjectClockDriver(obj.parent)
\ No newline at end of file
+        return getObjectClockDriver(obj.parent)
+    
+    
+def AbstractLogicInit(self, parent:Logic, name:str):
+    super(self.__class__, self).__init__(parent, name)
+    
+def AbstractLogic(class_name):
+    return type(class_name, # class name
+                (Logic,), # base classes
+                {'__init__': AbstractLogicInit} # methods
+                )
