@@ -2,13 +2,62 @@ import tkinter
 import tkinter.font
 from tkinter import *
 from tkinter import ttk
-
+from PIL import Image, ImageTk
+import os
 from .base import Logic
 from .schematic import Schematic
     
+import sys
+import importlib
+
+        
+def get_package_path(package_name):
+    if sys.version_info >= (3, 9):
+        import importlib.resources as resources
+        return resources.files(package_name)
+    else:
+        import importlib_resources as resources
+        return resources.path(package_name, '')
+
+def get_resource_path(package_name, resource_name):
+    if sys.version_info >= (3, 9):
+        import importlib.resources as resources
+        package_path = resources.files(package_name)
+        return package_path / resource_name
+    else:
+        import importlib.resources as resources
+        with resources.path(package_name, resource_name) as resource_path:
+            return resource_path
+    
+def getIcon(path):
+    #script_directory = os.path.dirname(os.path.abspath(__file__))
+    #icon_path = os.path.join(script_directory, path)
+    
+    icon = Image.open(path)
+    icon = ImageTk.PhotoImage(icon)
+    return icon
+
+def getResourceIcon(name):
+    path = get_resource_path('py4hw' , name)
+
+    return getIcon(path)
+
+
 class Workbench():
+    '''
+    This builds an interactive simulation GUI
     
+    +----------------------+----------------------------+
+    | Circuit Hierarchy    | Selected Circuit Interface |
+    | self.hierarchyTree   | self.interfacePane         |
+    |                      +----------------------------+
+    |                      | Selected Circuit Schematic |
+    |                      | self.schematicAreaPane     |
+    +----------------------+----------------------------+
     
+    '''
+    
+        
     def __init__(self, sys:Logic):
     
         self.sys = sys
@@ -25,14 +74,24 @@ class Workbench():
         
         self.topPane = PanedWindow(root, orient=HORIZONTAL)
         
-        self.hierarchyPane = PanedWindow(self.topPane, relief = SUNKEN, width=50, height=100)
+        self.hierarchyPane = Frame(self.topPane) # PanedWindow(self.topPane, relief = SUNKEN, width=50, height=100)
         self.topPane.add(self.hierarchyPane)
         
-        #print(ttk.Style().lookup("Prolepsis.Treeview", "font"))
+        self.buttonPane = Frame(self.hierarchyPane) #, relief = SUNKEN, width=100, height=100)
         
-        btnClock = ttk.Button(self.hierarchyPane, text="Clock", command=self.guiClk)
-        self.hierarchyPane.add(btnClock)
-        btnClock.pack()
+        self.txtClocks = tkinter.StringVar(value='1')
+        #print(ttk.Style().lookup("Prolepsis.Treeview", "font"))
+        txtClocks = ttk.Entry(self.buttonPane, width=10, textvariable=self.txtClocks)
+        #txtClocks.grid(row=0, column=0)
+        txtClocks.pack(side=tkinter.LEFT, padx=10, pady=(10,5))
+        
+        btnClock = ttk.Button(self.buttonPane, text="Clock", command=self.guiClk)
+        btnClock.pack(side=tkinter.RIGHT, padx=(10,0), pady=(10,5))
+        #btnClock.grid(row=0, column=1)
+        
+        #self.hierarchyPane.add(self.buttonPane)
+        
+        self.buttonPane.pack(side=tkinter.TOP)
         
         self.createHierarchyTree()
         
@@ -42,19 +101,23 @@ class Workbench():
         self.interfacePane = PanedWindow(self.rightPane, relief = SUNKEN, width=100, height=100)
         
         self.schematicAreaPane = PanedWindow(self.rightPane,  orient=VERTICAL, relief = SUNKEN, width=100, heigh=20)
-        self.schematicToolbarPane = PanedWindow(self.schematicAreaPane,  orient=HORIZONTAL, relief = SUNKEN, width=100, heigh=20)
+        self.schematicToolbarPane = Frame(self.schematicAreaPane) #,  orient=HORIZONTAL, relief = SUNKEN, width=100, heigh=20)
                 
         self.rightPane.add(self.interfacePane)
         self.rightPane.add(self.schematicAreaPane)
         self.schematicAreaPane.add(self.schematicToolbarPane) 
         
-        btnZoomOut = ttk.Button(self.schematicToolbarPane, text="Zoom out", command=self.guiZoomOut)
-        btnZoomIn = ttk.Button(self.schematicToolbarPane, text="Zoom in", command=self.guiZoomIn)
-
-        self.schematicToolbarPane.add(btnZoomOut)
-        self.schematicToolbarPane.add(btnZoomIn)
-
-#        btnZoomOut.pack()
+        
+        # Construct the full path to the image file
+        
+        icon_zo = getResourceIcon('zoomout24.png')
+        icon_zi = getResourceIcon('zoomin24.png')
+        
+        btnZoomIn = ttk.Button(self.schematicToolbarPane,  image=icon_zi, text="Zoom in", command=self.guiZoomIn)
+        btnZoomOut = ttk.Button(self.schematicToolbarPane, image=icon_zo, text="Zoom out", command=self.guiZoomOut)
+        
+        btnZoomIn.pack(side=tkinter.LEFT)
+        btnZoomOut.pack(side=tkinter.LEFT)
         
         self.schematicPane = PanedWindow(self.schematicAreaPane, relief = SUNKEN, width=100, height=100)
         self.schematicAreaPane.add(self.schematicPane)
@@ -65,14 +128,18 @@ class Workbench():
         
         self.topPane.pack(fill=BOTH, expand=True)
         
-        sys.getSimulator().addListener(self)
+        # explicitelly update
+        # sys.getSimulator().addListener(self)
         
         
         root.mainloop()
         
     def guiClk(self):
+        clks = int(self.txtClocks.get())
         sim = self.sys.getSimulator()
-        sim.clk(1)
+        sim.clk(clks)
+        #explicitelly update
+        self.simulatorUpdated()
 
     def guiZoomOut(self):
         self.schematicDiagram.scale("all", 0, 0, 0.9, 0.9)
@@ -111,6 +178,7 @@ class Workbench():
         
         detail_tv.config(columns=[ 'direction', 'width', 'value'])
         detail_tv.config(displaycolumns=[ 0, 1,2])
+        
 
         detail_tv.heading('#0', text='Name')
         detail_tv.heading('direction', text='Direction')
@@ -123,20 +191,59 @@ class Workbench():
         detail_tv.column('width', minwidth=0,width=100)
         detail_tv.column("value", minwidth=0,width=100)
         
-        detail_tv.config(selectmode=tkinter.NONE)
+        detail_tv.config(selectmode=tkinter.BROWSE)
         detail_tv.tag_configure("ally", background="green")
         #detail_tv.tag_bind("char", "<Double-Button-1>", event)
         #tv.config(style="Prolepsis.Treeview")
         detail_tv["style"] = "Prolepsis.Treeview"
 
-        self.debugTkinterHierarchy(self.root, 0)
+        detail_tv.bind("<Control-c>", self.copy_selected_text)
+        
+        #self.debugTkinterHierarchy(self.root, 0)
         
         if (obj == None):
             return
         
         self.setCircuitDetail(obj)
         
+    def copy_selected_text(self, event=None):
+        selected_items = self.detail_tv.selection() # Get IDs of all selected items
+
+        if not selected_items:
+            print("No row selected to copy.")
+            return
+
+        text_to_copy = []
+        for item_id in selected_items:
+            # Get the values for the current item (row)
+            # item_id is the internal identifier for the row
+            # 'values' returns a tuple of the data in each column for that row
+            row_values = self.detail_tv.item(item_id, 'values')
+
+            if row_values:
+                # Join the column values with a tab character.
+                # This is useful for pasting into spreadsheet applications.
+                text_to_copy.append("\t".join(map(str, row_values)))
+            else:
+                print(f"Warning: Selected item {item_id} has no values.")
+
+        if not text_to_copy:
+            print("No content found in selected rows to copy.")
+            return
+
+        # Clear the system clipboard
+        self.detail_tv.clipboard_clear()
+        # Append the combined text (rows separated by newlines) to the clipboard
+        self.detail_tv.clipboard_append("\n".join(text_to_copy))
+        # Force Tkinter to update the clipboard immediately
+        self.detail_tv.update()
+
+        print(f"Copied {len(selected_items)} row(s) to clipboard.")
+        # Optional: For debugging, you can print what's on the clipboard
+        # print(f"Clipboard content:\n{self.clipboard_get()}")
         
+
+         
     def debugTkinterHierarchy(self, obj, ii):
         indent = '|' * ii + '+'
         print('[INFO] {} tkinter obj {} {}'.format(indent, type(obj), obj._name if (hasattr(obj, '_name')) else ''));
@@ -155,12 +262,15 @@ class Workbench():
                 
             if (obj.isStructural()):
                 # ignore non structural circuits
-                sch = Schematic(obj, render='tkinter', parent=self.schematicPane)
+                sch = Schematic(obj, render='tkinter', parent=self.schematicPane, showValues=True)
     
                 sch.drawAll()
                 self.schematicDiagram = sch.canvas.canvas
                 self.schematicPane.add(self.schematicDiagram)
                 #self.schematicPane.pack()
+            elif (hasattr(obj, 'tkinter_gui')):
+                sch = obj.tkinter_gui(self.schematicPane)
+                self.schematicPane.add(sch)
             
         self.detailObj = obj
         self.detail_tv.delete(*self.detail_tv.get_children())
@@ -233,9 +343,14 @@ class Workbench():
         tv = self.hierarchyTree  
         self.map_id_obj = {}
     
+        
         tv.bind('<<TreeviewSelect>>', self.callback)
-        self.hierarchyPane.add(tv)
+        #self.hierarchyPane.add(tv)
         tv.pack(fill=BOTH, expand=YES)
+        #tv.grid(row=1, column=0, columnspan=2, sticky='nsew')
+        
+        
+        
         dc_iid = tv.insert("", tkinter.END, text="HWSystem", values=['Top', 'Top level'], open=True)
         
         self.map_id_obj[dc_iid] = self.sys
