@@ -8,7 +8,9 @@ Created on Wed Jan 19 12:40:41 2022
 from .bitwise import *
 from .arithmetic import *
 from .arithmetic_fp import *
+from .arithmetic_fxp import *
 from .relational import *
 from .simulation import *
 from .clock import *
+from .storage import *
 from .. import *
\ No newline at end of file
