@@ -6,6 +6,7 @@ Created on Wed Jan 19 12:53:47 2022
 """
 from .. import *
 from .bitwise import *
+
 from deprecated import deprecated
 
 
@@ -14,14 +15,57 @@ class Add(Logic):
     Combinational Arithmetic Add
     """
 
-    def __init__(self, parent, name: str, a: Wire, b: Wire, r: Wire):
+    def __init__(self, parent, name: str, a: Wire, b: Wire, r: Wire, ci=None, co=None):
         super().__init__(parent, name)
         self.a = self.addIn("a", a)
         self.b = self.addIn("b", b)
         self.r = self.addOut("r", r)
+        
+        if (ci is None):
+            ci = self.wire('ci')
+            Constant(self, 'ci', 0, ci)
+            
+        else:
+            self.ci = self.addIn('ci', ci)
+            
+        aw = a.getWidth()
+        bw = b.getWidth()
+        rw = r.getWidth()
+        
+        if (co is None):
+            assert((rw  - max(aw,bw)) in [0,1])
+            AddCarryIn(self, "add", a, b, r, ci)
+            
+        else:
+            assert(rw  == max(aw,bw) )
+            self.co = self.addOut('co', co)
+            pre_r = self.wire('pre_r', rw + 1)
+            AddCarryIn(self, "add", a, b, pre_r, ci)
+            Range(self, 'r', pre_r, rw-1, 0, r)
+            Bit(self, 'co', pre_r, rw, co)
+            
+        
+
+        
+        
+class AddCarryIn(Logic):
+    """
+    Combinational Arithmetic Add
+    """
+
+    def __init__(self, parent, name: str, a: Wire, b: Wire, r: Wire, ci:Wire):
+        super().__init__(parent, name)
+        self.a = self.addIn("a", a)
+        self.b = self.addIn("b", b)
+        self.r = self.addOut("r", r)
+        
+        assert(r.getWidth() >= a.getWidth() )
+        self.ci = self.addIn('ci', ci)
+        
 
     def propagate(self):
-        self.r.put(self.a.get() + self.b.get())
+        self.r.put(self.a.get() + self.b.get() + self.ci.get())        
+
 
 class Abs(Logic):
     """
@@ -125,8 +169,6 @@ class ZeroExtend(Logic):
 
 
 
-
-
 class Mul(Logic):
     """
     Combinational Arithmetic Multiplier
@@ -141,7 +183,25 @@ class Mul(Logic):
     def propagate(self):
         self.r.put(self.a.get() * self.b.get())
 
+class SignedMul(Logic):
+    """
+    Arithmetic Signed Multiplier
+    """
+    def __init__(self, parent, name: str, a: Wire, b: Wire, r: Wire):
+        super().__init__(parent, name)
+        self.a = self.addIn("a", a)
+        self.b = self.addIn("b", b)
+        self.r = self.addOut("r", r)
 
+    def propagate(self):
+        from ..helper import IntegerHelper    
+        
+        sa = IntegerHelper.c2_to_signed(self.a.get(), self.a.getWidth())
+        sb = IntegerHelper.c2_to_signed(self.b.get(), self.b.getWidth())
+        mask = (1 << self.r.getWidth()) - 1
+        newValue = (sa * sb) & mask
+        self.r.put(newValue)
+        
 class Div(Logic):
     """
     Combinational Arithmetic Divider
@@ -154,7 +214,31 @@ class Div(Logic):
         self.r = self.addOut("r", r)
 
     def propagate(self):
-        self.r.put(self.a.get() // self.b.get())
+        # @todo verilog implementation of div is unpredictable
+        import random
+        if (self.b.get() == 0):
+            self.r.put(random.randint(0, 1<<self.r.getWidth()))
+        else:
+            self.r.put(self.a.get() // self.b.get())
+
+class Mod(Logic):
+    """
+    Combinational Arithmetic Modulo (reminder of div)
+    """
+
+    def __init__(self, parent, name: str, a: Wire, b: Wire, r: Wire):
+        super().__init__(parent, name)
+        self.a = self.addIn("a", a)
+        self.b = self.addIn("b", b)
+        self.r = self.addOut("r", r)
+
+    def propagate(self):
+        # @todo verilog implementation of div is unpredictable
+        import random
+        if (self.b.get() == 0):
+            self.r.put(random.randint(0, 1<<self.r.getWidth()))
+        else:
+            self.r.put(self.a.get() % self.b.get())
 
 
 class SignedDiv(Logic):
@@ -194,7 +278,7 @@ class SignedDiv(Logic):
 
 class Sub(Logic):
     """
-    Arithmetic Add
+    Arithmetic Sub
     """
 
     def __init__(self, parent, name: str, a: Wire, b: Wire, r: Wire):
@@ -209,7 +293,27 @@ class Sub(Logic):
         self.r.put(newValue)
 
 
+class SignedSub(Logic):
+    """
+    Arithmetic Sub
+    """
+    
 
+    def __init__(self, parent, name: str, a: Wire, b: Wire, r: Wire):
+        super().__init__(parent, name)
+        self.a = self.addIn("a", a)
+        self.b = self.addIn("b", b)
+        self.r = self.addOut("r", r)
+
+    def propagate(self):
+        from ..helper import IntegerHelper    
+        
+        sa = IntegerHelper.c2_to_signed(self.a.get(), self.a.getWidth())
+        sb = IntegerHelper.c2_to_signed(self.b.get(), self.b.getWidth())
+        mask = (1 << self.r.getWidth()) - 1
+        newValue = (sa - sb) & mask
+        self.r.put(newValue)
+        
 class Counter(Logic):
     """
     Counts up to the value mod and returns to zero
@@ -222,8 +326,11 @@ class Counter(Logic):
         from .bitwise import Mux2
         from .storage import Reg
         
-        reset = self.addIn('reset', reset)
-        inc = self.addIn('inc', inc)
+        if not(reset is None):
+            reset = self.addIn('reset', reset)
+        if not(inc is None):
+            inc = self.addIn('inc', inc)
+        
         q = self.addOut('q', q)
     
         one = self.wire('one', q.getWidth())
@@ -236,6 +343,11 @@ class Counter(Logic):
         Constant(self, 'one', 1, one)
         Constant(self, 'zero', 0, zero)
         
+        if (inc is None):
+            inc = one
+        if (reset is None):
+            reset = zero
+            
         Mux2(self, 'muxinc', inc, q, add, d1)
         Mux2(self, 'muxreset', reset, d1, zero, d)
 
@@ -287,24 +399,58 @@ class ModuloCounter(Logic):
         
         
 class ShiftRight(Logic):
-    def __init__(self, parent:Logic, name:str, a, b, r):
+    def __init__(self, parent:Logic, name:str, a, b, r, arithmetic=False):
         super().__init__(parent, name)
 
         a = self.addIn('a', a)
         b = self.addIn('b', b)
         r = self.addOut('r', r)
-            
+        
+        last = a
+        w = last.getWidth()
         wb = b.getWidth()
         
-        num = int(math.pow(2, wb))
-        ins = []
-        
-        for i in range(num):
-            shifted = self.wire('shifted{}'.format(i), r.getWidth())
-            ShiftRightConstant(self, 'shifted{}'.format(i), a, i, shifted)
-            ins.append(shifted)
+        if (isinstance(arithmetic, Wire)):
+            self.addIn('arithmetic', arithmetic)
             
-        Mux(self, 'mux', b, ins, r)
+            signExtended = self.wire(f'sign_extended', w + (1<<wb))
+            SignExtend(self, f'sign_extended', last, signExtended)
+            
+            zeroExtended = self.wire(f'zero_extended', w + (1<<wb))
+            ZeroExtend(self, f'zero_extended', last, zeroExtended)
+
+            last = self.wire(f'extended', w + (1<<wb))
+            
+            Mux2(self, 'extended', arithmetic, zeroExtended, signExtended, last)
+            w = last.getWidth()           
+            
+        else:
+            if (arithmetic):
+                #
+                signExtended = self.wire(f'sign_extended', w + (1<<wb))
+                SignExtend(self, f'sign_extended', last, signExtended)
+                last = signExtended
+                w = last.getWidth()
+            else:
+                pass
+            
+
+        if (wb > 6):
+            print('WARNING shift registers with shifting value width > 5 are not common')
+        
+        
+        for i in range(wb):
+            shifted = self.wire('shifted{}'.format(i), w )
+            
+            ShiftRightConstant(self, 'shifted{}'.format(i), last, 1<<i, shifted)
+            
+            doShift = self.wire(f'doShift{i}')
+            Bit(self, f'doShift{i}', b, i, doShift)
+            prer = self.wire(f'shift_{i}', w)
+            Mux2(self, f'shift_{i}', doShift, last, shifted, prer)
+            last = prer
+            
+        Buf(self, 'r', prer, r)
 
 class ShiftLeft(Logic):
     def __init__(self, parent:Logic, name:str, a, b, r):
@@ -314,17 +460,83 @@ class ShiftLeft(Logic):
         b = self.addIn('b', b)
         r = self.addOut('r', r)
             
+        wa = a.getWidth()
+        wb = b.getWidth()
+        wr = r.getWidth()
+        
+        w = max(wa, wr)
+        
+        if (wb > 6):
+            print('WARNING shift registers with shifting value width > 5 are not common')
+            
+        last = a
+        for i in range(wb):
+            shifted = self.wire('shifted{}'.format(i), w)
+            ShiftLeftConstant(self, 'shifted{}'.format(i), last, 1<<i, shifted)
+            
+            doShift = self.wire(f'doShift{i}')
+            Bit(self, f'doShift{i}', b, i, doShift)
+            prer = self.wire(f'shift_{i}', w)
+            Mux2(self, f'shift_{i}', doShift, last, shifted, prer)
+            last = prer
+            
+        Buf(self, 'r', prer, r)
+
+class RotateRight(Logic):
+    def __init__(self, parent:Logic, name:str, a, b, r):
+        super().__init__(parent, name)
+
+        a = self.addIn('a', a)
+        b = self.addIn('b', b)
+        r = self.addOut('r', r)
+            
+        w = a.getWidth()
         wb = b.getWidth()
         
-        num = int(math.pow(2, wb))
-        ins = []
-        
-        for i in range(num):
-            shifted = self.wire('shifted{}'.format(i), r.getWidth())
-            ShiftLeftConstant(self, 'shifted{}'.format(i), a, i, shifted)
-            ins.append(shifted)
+        if (wb > 6):
+            print('WARNING shift registers with shifting value width > 5 are not common')
             
-        Mux(self, 'mux', b, ins, r)
+        last = a
+        for i in range(wb):
+            shifted = self.wire('shifted{}'.format(i), r.getWidth())
+            RotateRightConstant(self, 'shifted{}'.format(i), last, 1<<i, shifted)
+            
+            doShift = self.wire(f'doShift{i}')
+            Bit(self, f'doShift{i}', b, i, doShift)
+            prer = self.wire(f'shift_{i}', a.getWidth())
+            Mux2(self, f'shift_{i}', doShift, last, shifted, prer)
+            last = prer
+            
+        Buf(self, 'r', prer, r)
+
+
+class RotateLeft(Logic):
+    def __init__(self, parent:Logic, name:str, a, b, r):
+        super().__init__(parent, name)
+
+        a = self.addIn('a', a)
+        b = self.addIn('b', b)
+        r = self.addOut('r', r)
+            
+        w = a.getWidth()
+        wb = b.getWidth()
+        
+        if (wb > 6):
+            print('WARNING shift registers with shifting value width > 5 are not common')
+            
+        last = a
+        for i in range(wb):
+            shifted = self.wire('shifted{}'.format(i), r.getWidth())
+            RotateLeftConstant(self, 'shifted{}'.format(i), last, 1<<i, shifted)
+            
+            doShift = self.wire(f'doShift{i}')
+            Bit(self, f'doShift{i}', b, i, doShift)
+            prer = self.wire(f'shift_{i}', a.getWidth())
+            Mux2(self, f'shift_{i}', doShift, last, shifted, prer)
+            last = prer
+            
+        Buf(self, 'r', prer, r)
+
         
 class BinaryToBCD(Logic):
     """
@@ -332,12 +544,36 @@ class BinaryToBCD(Logic):
     """
     
     def __init__(self, parent, name : str, a: Wire, r:Wire):
+        from ..helper import LogicHelper    
+
         super().__init__(parent, name)
         
         a = self.addIn('a', a)
         r = self.addOut('r', r)
     
-        raise Exception('Not implemented')
+        hlp = LogicHelper(self)
+        
+        w = a.getWidth()
+        assert(r.getWidth() % 4 == 0)
+        digits = r.getWidth() // 4 # int(math.ceil(math.log10((2**w)-1)))
+        print('Number of BCD digits:', digits)
+        print('r width:', r.getWidth())
+        
+        assert(r.getWidth() >= (digits*4))
+        
+        ret = []
+        v = a
+        k10 = hlp.hw_constant(4, 10)
+        
+        for i in range(digits):
+            rem = self.wire('mod{}'.format(i), 4)
+            div = self.wire('div{}'.format(i), w)
+            Mod(self, 'mod{}'.format(i), v, k10, rem)
+            Div(self, 'div{}'.format(i), v, k10, div)
+            ret.append(rem)
+            v = div
+            
+        ConcatenateLSBF(self, 'r', ret, r)
         
 
 class _FFunction(Logic):
@@ -403,7 +639,33 @@ class CountLeadingZeros(Logic):
     https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=4539802    
     DOI: 10.1109/TVLSI.2008.2000458
     """
-    def __init__(self, parent, name : str, a: Wire, r:Wire, z:Wire):
+    def __init__(self, parent:Logic, name : str, a: Wire, r:Wire, z:Wire):
+        '''
+        
+
+        Parameters
+        ----------
+        parent : Logic
+            Parent circuit.
+        name : str
+            Instance name.
+        a : Wire
+            Input wire with the .
+        r : Wire
+            Number of leading zeros in the input wire.
+        z : Wire
+            1 if input was zero.
+
+        Raises
+        ------
+        Exception
+            DESCRIPTION.
+
+        Returns
+        -------
+        None.
+
+        '''
         super().__init__(parent, name)
         
         a = self.addIn('a', a)
