@@ -44,13 +44,14 @@ class And(Logic):
         
         num = len(ins)
 
+        if (num == 1):
+            Buf(self, 'and1', ins[0], r)
+            return
+        
         if (num == 2):
             And2(self, 'and2', ins[0], ins[1], r)
             return
             
-        if (num < 3):
-            raise Exception('List should be > 2')
-
         # by now we do an inefficient ladder structure, we should
         # do a more fancy logarithmic design
         
@@ -154,6 +155,45 @@ class Buf(Logic):
     def propagate(self):
         self.r.put(self.a.get())
 
+    
+class BidirBuf(Logic):
+    
+    def __init__(self, parent, name: str, pin : Wire , pout : Wire, poe : Wire, bidir : BidirWire):
+        super().__init__(parent, name)
+        
+        if (poe.getWidth() != 1):
+            raise Exception('BidirBuf only supports 1 bit output enable')
+            
+        self.bidir = self.addInOut('bidir', bidir)
+        self.pin = self.addOut('pin', pin)
+        self.pout = self.addIn('pout', pout)
+        self.poe = self.addIn('poe', poe)
+        
+    def propagate(self):
+        if (self.poe.get() ==1):
+            self.bidir.put(self.pout.get())
+        else:
+            self.pin.put(self.bidir.get())
+
+class BufEnable(Logic):
+    # This is just a way to control set to zero all bits of a signal
+    # depending on an enable signal
+    def __init__(self, parent, name, a, en, r):
+        super().__init__(parent, name)
+        
+        self.a = self.addIn('a', a)
+        self.en = self.addIn('en', en)
+        self.r = self.addOut('r', r)
+   
+        assert(a.getWidth() == r.getWidth())
+    
+        re = self.wire('re', r.getWidth())
+        Repeat(self, 're', en, re)
+        And2(self, 'r', a, re, r)
+        
+    def structureName(self):
+        return 'BufEnable{}'.format(self.a.getWidth())
+    
 
 class Constant(Logic):
     """
@@ -162,6 +202,8 @@ class Constant(Logic):
 
     def __init__(self, parent: Logic, name: str, value: int, r: Wire):
         super().__init__(parent, name)
+        assert(isinstance(value, int))
+        assert(isinstance(r, Wire))
         self.value = value
         self.r = self.addOut("r", r)
 
@@ -169,6 +211,23 @@ class Constant(Logic):
         self.r.put(self.value)
         #print(self.name, '=', self.value)
         
+class Demux(Logic):
+    def __init__(self, parent, name, a, sel, r):
+        super().__init__(parent, name)
+        
+        self.addIn('a', a)
+        self.addIn('sel', sel)
+        
+        assert(2**sel.getWidth() == len(r))
+        
+        ss = self.wires('ss', 2**sel.getWidth(), 1)
+        Decoder(self, 'decoder', sel, ss)
+        
+        for idx, ri in enumerate(r):
+            self.addOut('r{}'.format(idx), ri)
+            
+            BufEnable(self, 'en{}'.format(idx), a, ss[idx], ri)
+            
 class Nand2(Logic):
     """
     Binary Nand
@@ -253,6 +312,17 @@ class Or2(Logic):
     def propagate(self):
         self.r.put(self.a.get() | self.b.get())
 
+class OrBits(Logic):
+    def __init__(self, parent, name:str, a: Wire, r: Wire):
+        super().__init__(parent, name)
+        
+        self.addIn('a', a)
+        self.addOut('r', r)
+        
+        ab = self.wires('ab', a.getWidth(), 1)
+        BitsLSBF(self, 'bits', a, ab)
+        Or(self, 'or', ab, r)
+    
 class Or(Logic):
     def __init__(self, parent, name:str, ins, r: Wire):
         super().__init__(parent, name)
@@ -269,13 +339,14 @@ class Or(Logic):
         
         num = len(ins)
 
+        if (num == 1):
+            Buf(self, 'or1', ins[0], r)
+            return 
+        
         if (num == 2):
             Or2(self, 'or2', ins[0], ins[1], r)
             return
-            
-        if (num < 3):
-            raise Exception('List should be > 2')
-
+                    
         # by now we do an inefficient ladder structure, we should
         # do a more fancy logarithmic design
         
@@ -299,13 +370,23 @@ class ShiftLeftConstant(Logic):
         super().__init__(parent, name)
         self.a = self.addIn('a', a)
         self.r = self.addOut('r', r)
-        self.n = n
+        self.addParameter('n', n)
 
     def propagate(self):
-        self.r.put(self.a.get() << self.n)
+        self.r.put(self.a.get() << self.getParameterValue('n'))
 
 
 class ShiftRightConstant(Logic):
+    def __init__(self, parent, name: str, a: Wire, n: int, r: Wire):
+        super().__init__(parent, name)
+        self.a = self.addIn('a', a)
+        self.r = self.addOut('r', r)
+        self.addParameter('n', n)
+
+    def propagate(self):
+        self.r.put(self.a.get() >>  self.getParameterValue('n'))
+
+class RotateLeftConstant(Logic):
     def __init__(self, parent, name: str, a: Wire, n: int, r: Wire):
         super().__init__(parent, name)
         self.a = self.addIn('a', a)
@@ -313,7 +394,24 @@ class ShiftRightConstant(Logic):
         self.n = n
 
     def propagate(self):
-        self.r.put(self.a.get() >> self.n)
+        a = self.a.get()
+        n = self.n
+        w = self.a.getWidth()
+        self.r.put((a << n) | (a >> (w - n)))
+
+
+class RotateRightConstant(Logic):
+    def __init__(self, parent, name: str, a: Wire, n: int, r: Wire):
+        super().__init__(parent, name)
+        self.a = self.addIn('a', a)
+        self.r = self.addOut('r', r)
+        self.n = n
+
+    def propagate(self):
+        a = self.a.get()
+        n = self.n
+        w = self.a.getWidth()
+        self.r.put((a >> n) | (a << (w - n)))
                 
 class Xor2(Logic):
     """
@@ -454,7 +552,7 @@ class Repeat(Logic):
         else:
             self.r.put(wf)
 
-
+# deprecated use OneHotMux
 class Select(Logic):
     def __init__(self, parent, name: str, sels:list, ins:list, r: Wire):
         super().__init__(parent, name)
@@ -476,8 +574,48 @@ class Select(Logic):
         Or(self, 'or', final, r)
 
 
+class OneHotMux(Logic):
+    def __init__(self, parent, name: str, sels:list, ins:list, r: Wire):
+        super().__init__(parent, name)
+
+        final = []
+
+        for idx, sel in enumerate(sels):
+            inv = ins[idx]
+            
+            sel_name = sel.name
+            in_name = inv.name 
+            self.addIn(sel_name, sel)
+            self.addIn(in_name, inv)
+            selx = self.wire('selx{}'.format(idx), inv.getWidth())
+            and_sel = self.wire('and_sel{}'.format(idx), inv.getWidth())
+
+            Repeat(self, 'sel{}'.format(idx), sel, selx)
+            And2(self, 'and{}'.format(idx), selx, inv, and_sel)
+            final.append(and_sel)
+
+        self.addOut('r', r)
+        Or(self, 'or', final, r)
 
 
+class OneHotDemux(Logic):
+    def __init__(self, parent, name: str, sels:list, a:Wire, outs: list):
+        super().__init__(parent, name)
+
+        final = []
+
+        a = self.addIn('a', a)
+
+        for idx, sel in enumerate(sels):
+            self.addIn('sel{}'.format(idx), sel)
+            self.addOut('out{}'.format(idx), outs[idx])
+            
+            selx = self.wire('selx{}'.format(idx), a.getWidth())
+
+            Repeat(self, 'sel{}'.format(idx), sel, selx)
+            And2(self, 'and{}'.format(idx), selx, a, outs[idx])
+            
+        
 class SelectDefault(Logic):
     def __init__(self, parent, name, sels, ins, default, r):
         super().__init__(parent, name)
@@ -577,6 +715,25 @@ class Minterm(Logic):
         And(self, 'prod', parts, r)
         
 
+class SumOfMinterms(Logic):
+    def __init__(self, parent, name, a, minterms:list, r):
+        super().__init__(parent, name)
+        
+        self.addIn('a', a)
+        self.addOut('r', r)
+        
+        bits = self.wires('bits', a.getWidth(), 1)
+        BitsLSBF(self, 'bits', a, bits)
+        
+        acum = []
+        for idx, minterm in enumerate(minterms):
+            v = self.wire('s{}'.format(idx))
+            
+            Minterm(self, 'minterm{}'.format(idx), bits, minterm, v)
+            acum.append(v)
+            
+        Or(self, 'sum', acum, r)
+        
 class ConcatenateMSBF(Logic):
     """
     Concatenate wires circuit in MSBF order
@@ -656,3 +813,78 @@ class Range(Logic):
         newvalue = (value >> self.low) & mask
         self.r.put(newvalue)
 
+class Digit7Segment(Logic):
+    def __init__(self, parent, name, v, led):
+        super().__init__(parent, name)
+        
+        assert(led.getWidth() == 7)
+        
+        self.addIn('v', v)
+        self.addOut('led', led)
+        
+        
+        a_minterms = [0,2,3,5,6,7,8,9,0xA,0xC,0xE,0xF]
+        b_minterms = [0,1,2,3,4,7,8,9,0xA,0xd]
+        c_minterms = [0,1,3,4,5,6,7,8,9,0xA,0xb,0xd]
+        d_minterms = [0,2,3,5,6,8,0xb,0xC,0xd,0xE]
+        e_minterms = [0,2,6,8,0xA,0xb,0xC,0xd,0xE,0xF]
+        f_minterms = [0,4,5,6,8,9,0xA,0xb,0xC,0xE,0xF]
+        g_minterms = [2,3,4,5,6,8,9,0xA,0xb,0xd,0xE,0xF]
+        
+        
+        a = self.wire('a')
+        b = self.wire('b')
+        c = self.wire('c')
+        d = self.wire('d')
+        e = self.wire('e')
+        f = self.wire('f')
+        g = self.wire('g')
+        
+        na = self.wire('na')
+        nb = self.wire('nb')
+        nc = self.wire('nc')
+        nd = self.wire('nd')
+        ne = self.wire('ne')
+        nf = self.wire('nf')
+        ng = self.wire('ng')
+        
+        ConcatenateLSBF(self, 'led', [a,b,c,d,e,f,g], led)
+        
+        SumOfMinterms(self, 'a', v, a_minterms, a)
+        SumOfMinterms(self, 'b', v, b_minterms, b)
+        SumOfMinterms(self, 'c', v, c_minterms, c)
+        SumOfMinterms(self, 'd', v, d_minterms, d)
+        SumOfMinterms(self, 'e', v, e_minterms, e)
+        SumOfMinterms(self, 'f', v, f_minterms, f)
+        SumOfMinterms(self, 'g', v, g_minterms, g)
+        
+class PriorityEncoder(Logic):
+    def __init__(self, parent, name, a, r, inc_priority=True):
+        from ..helper import LogicHelper
+        super().__init__(parent, name)
+        assert(len(a) == len(r))
+
+        # Make a copy so that reverse does not affect the original list
+        a = a.copy()
+        r = r.copy()
+        
+        # It the priority is decreasing, the a[0] is the more priorized
+        # Otherwise is the higher index
+        if (inc_priority):
+            a.reverse()
+            r.reverse()
+        
+        hlp = LogicHelper(self)
+        
+        last = None
+        
+        for i in range(len(a)):
+            self.addIn('a{}'.format(i), a[i])
+            self.addOut('r{}'.format(i), r[i])
+            
+            if (last is None):
+                Buf(self, 'r{}'.format(i), a[i], r[i])
+                last = hlp.hw_buf(a[i])
+            else:
+                And2(self, 'r{}'.format(i), a[i], hlp.hw_not(last), r[i])
+                last = hlp.hw_or2(last, a[i])
\ No newline at end of file
