@@ -15,10 +15,25 @@ import tkinter.font
 from tkinter import *
 from tkinter import ttk
     
+class FieldInspector:
+    def __init__(self, obj, field):
+        self.obj = obj
+        self.name = field
+        
+    def getFormat(self):
+        return '{}'
+    
+    def get(self):
+        return getattr(self.obj, self.name)
+    
+    def getFullPath(self):
+        return self.obj.getFullPath() + '/' + self.name
+        
 class Waveform(Logic):
     def __init__(self, parent, name, wires):
         """
-        
+        Collects the progress of signals along the time.
+        We try to reduce data by avoiding data duplication.
 
         Parameters
         ----------
@@ -35,28 +50,49 @@ class Waveform(Logic):
 
         """
         super().__init__(parent, name)
+        
+        # wires contain all the elements of the input list, there might be repeatitions
         self.wires = wires if isinstance(wires, list) else [wires]
-        self.data = {}  # this is a dictionary of obj (wire or port) -> data, list of wire values 
+        self.format = []
+        
+        
+        self.data = {}  
+        # this is a dictionary of obj (wire or port) -> data, list of wire values 
+        # so, repetitions in wires will be discarded
 
-        uniqueWires = []
+        self.uniqueWires = []
         
         for x in self.wires:
             
             if isinstance(x, Wire):
-                if not(x in uniqueWires):
+                w = x
+                if not(x in self.uniqueWires):
                     self.addIn(x.name, x)
-                    uniqueWires.append(x)
+                    self.uniqueWires.append(x)
+                    self.data[x] = []            
             elif isinstance(x, InPort) or isinstance(x, OutPort):
                 w = x.wire
                 if (w is None):
                     raise Exception('Wire is null for port', x.getFullPath())
-                if not (w in uniqueWires):
+                if not (w in self.uniqueWires):
                     self.addIn(w.name, w)
-                    uniqueWires.append(w)
+                    self.uniqueWires.append(w)
+                    self.data[w] = []     
+            elif isinstance(x, FieldInspector):
+                w = None
+                if not(x in self.uniqueWires):
+                    self.uniqueWires.append(x)
+                    self.data[x] = []
             else:
-                raise Exception('Unsupported object class to watch ' + type(x))
+                raise Exception('Unsupported object class to watch: {}'.format(type(x)))
                 
-            self.data[x] = []            
+
+            if isinstance(x, FieldInspector):
+               self.format.append(x.getFormat()) 
+            elif (w.getWidth() == 1):
+                self.format.append('')
+            else:
+                self.format.append('{:X}')
 
     @staticmethod
     def getwire(x):
@@ -69,21 +105,42 @@ class Waveform(Logic):
             
         return w
     
+    def clear(self):
+        for key in self.data.keys():
+            self.data[key] = []
+        
+        
     def clock(self):
-        for x in self.wires:
-            w = Waveform.getwire(x)
+        # Data is indexed by wire (or field inspector) to avoid repeats
+        # For ports, the value used for x will be its wire
+        for x in self.uniqueWires:
+            if isinstance(x, FieldInspector):
+                self.data[x].append(x.get())
+            else:
+                w = Waveform.getwire(x)
                 
-            self.data[x].append(w.get())
+                if not(w in self.data):
+                    raise Exception('Wire {}/{} not in data list {}'.format(x, w, self.data.keys()))
+                    
+                self.data[w].append(w.get())
             
     def getDict(self):
         return self.data
 
     def draw(self):
         import matplotlib.pyplot as plt
-        fig, axs = plt.subplots(len(self.data.keys())+1, sharex=True)
+        fig, axs = plt.subplots(len(self.wires)+1, sharex=True)
+        
+        obj = self.wires[0]
+        if (isinstance(obj, FieldInspector)):
+            w = obj
+            ww = -1
+        else:
+            w = Waveform.getwire(obj)
+            ww = w.getWidth()
         
         
-        numclocks = len(self.data[self.wires[0]])
+        numclocks = len(self.data[w])
         clock = 1 - np.arange(numclocks*2) % 2
         tclock = 0.5 * np.arange(numclocks*2)
         t =  np.arange(numclocks)
@@ -91,12 +148,16 @@ class Waveform(Logic):
         axs[0].step(tclock, clock, 'r', linewidth = 2, where='post')
         axs[0].set_ylabel('clock', rotation=0)
         
-        for idx, x in enumerate(self.wires):
-           axs[idx+1].step(t, self.data[x], linewidth=2, where='post')
-           axs[idx+1].set_ylabel(x.name, rotation=0)
+        for idx, obj in enumerate(self.wires):
+            w = Waveform.getwire(obj)
+
+            axs[idx+1].step(t, self.data[w], linewidth=2, where='post')
+            axs[idx+1].set_ylabel(w.name, rotation=0)
         
         return fig, axs
     
+
+    
     def draw_wavedrom(self, shortNames=False):
         """
         
@@ -116,6 +177,8 @@ class Waveform(Logic):
         import nbwavedrom as wave
         return wave.draw(self.get_wavedrom(shortNames))
     
+    
+    
     def get_wavedrom(self, shortNames=False):
         """
         
@@ -134,22 +197,29 @@ class Waveform(Logic):
         
         signals = [{"name": "clk", 'wave': 'P'}]
         
-        for obj in self.wires:
-            w = Waveform.getwire(obj)
+        for idx, obj in enumerate(self.wires):
+            if (isinstance(obj, FieldInspector)):
+                w = obj
+                ww = -1
+            else:
+                w = Waveform.getwire(obj)
+                ww = w.getWidth()
+                
+            fmt = self.format[idx]
             wavedata = 'x'
             wavedatadata = []
             
-            data = self.data[obj]
+            data = self.data[w]
             last = 'x'
             numclks = len(data)
             for i in range(numclks):
                 v = data[i]
                 if (v != last):
-                    if (w.getWidth() == 1):
+                    if (ww == 1):
                         wavedata += '{}'.format(v)
                     else:
                         wavedata += '{}'.format(2)
-                        wavedatadata.append( '{:X}'.format(v))
+                        wavedatadata.append(fmt.format(v))
                 else:
                     wavedata += '.'
                 last = v
@@ -180,34 +250,44 @@ class Waveform(Logic):
         
         return ret
     
-    def gui(self, root=None):
+    def gui(self, root=None, shortNames=False):
         
-        window = WaveformWindow(root, self)
+        window = WaveformWindow(root, self, shortNames)
 
 class WaveformWindow:
     
-    def __init__(self, root, wvf):
+    def __init__(self, root, wvf, shortNames):
+        from py4hw.gui import getResourceIcon 
         if (root is None):
             self.root = tkinter.Tk()
         else:
             self.root = root
 
         self.waveform = wvf
+        self.shortNames = shortNames
             
         self.root.title('Waveform viewer ' + wvf.name)
         
-        ttk.Style().configure("Treeview", fg="light yellow")
-        font = tkinter.font.Font(size=8)
-        ttk.Style().configure("Prolepsis.Treeview", font=font)
-
+        font = tkinter.font.Font(size=9)
+        
+        style = ttk.Style()
+        style.configure("Treeview", fg="light yellow")
+        style.configure("Treeview", rowheight=21) 
+        style.configure("Prolepsis.Treeview", font=font)
+        
         self.topPane = PanedWindow(self.root, orient=VERTICAL)
 
         self.toolbar= Frame(self.topPane, bd=1, relief=RAISED)
         
-        zoomInButton = Button(self.toolbar, text='zoomIn', relief=FLAT,  command=self.zoomIn)
-        zoomInButton.pack(side=LEFT, padx=2, pady=2)
-        zoomOutButton = Button(self.toolbar, text='zoomOut', relief=FLAT,  command=self.zoomOut)
-        zoomOutButton.pack(side=LEFT, padx=2, pady=2)
+        icon_zo = getResourceIcon('zoomout24.png')
+        icon_zi = getResourceIcon('zoomin24.png')
+        
+
+        zoomInButton = ttk.Button(self.toolbar,  image=icon_zi, text="Zoom in", command=self.zoomIn)
+        zoomOutButton = ttk.Button(self.toolbar, image=icon_zo, text="Zoom out", command=self.zoomOut)
+
+        zoomInButton.pack(side=tkinter.LEFT)
+        zoomOutButton.pack(side=tkinter.LEFT)
 
         self.topPane.add(self.toolbar)
 
@@ -224,14 +304,23 @@ class WaveformWindow:
 
         self.linesPane = PanedWindow(self.mainPane, relief = SUNKEN, width=100, height=100)
         self.mainPane.add(self.linesPane)
+
+        self.hclock = 20
+        self.hw = self.hclock * self.getNumClocks()
         
-        self.canvas = Canvas(self.linesPane, bg='white', scrollregion=(0,0,1000,1000))
+        self.canvas = Canvas(self.linesPane, bg='white', scrollregion=(0,0, self.hw,1000))
         #self.canvas.configure(yscrollcommand=self.scroll.set)
         self.canvas.configure(yscrollincrement='20')
         
+        self.h_scroll = ttk.Scrollbar(self.linesPane, orient=HORIZONTAL, command=self.canvas.xview)
+        self.v_scroll = ttk.Scrollbar(self.linesPane, orient=VERTICAL, command=self.sync_scrolls)
+        self.canvas.configure(xscrollcommand=self.h_scroll.set, yscrollcommand=self.v_scroll.set)
+
+        
         self.linesPane.add(self.canvas)
+        self.h_scroll.pack(side=tkinter.BOTTOM, fill=X)
+        self.v_scroll.pack(side=tkinter.RIGHT, fill=Y)
 
-        self.hclock = 20
         self.drawWaveforms()
 
         self.topPane.pack(fill=BOTH, expand=True)
@@ -243,12 +332,23 @@ class WaveformWindow:
 
     def zoomIn(self):
         self.hclock *= 1.1
+        self.hw = self.hclock * self.getNumClocks()
+
+        self.canvas.config(scrollregion=(0, 0, self.hw, 1000))
         self.redraw()
     
     def zoomOut(self):
         self.hclock /= 1.1
+        self.hw = self.hclock * self.getNumClocks()
+
+        self.canvas.config(scrollregion=(0, 0, self.hw, 1000))
         self.redraw()
     
+    def sync_scrolls(self, *args):
+        self.hierarchyTree.yview(*args)
+        self.v_scroll.set(*args)
+        self.canvas.yview_moveto(args[0])
+    
     def yview(self, *args):
         print(*args)
         self.hierarchyTree.yview(*args)
@@ -274,10 +374,16 @@ class WaveformWindow:
         #self.hierarchyPane.pack(fill=BOTH, expand=YES)
         tv.pack(side=LEFT, fill=BOTH, expand=YES)
         
-        wd = self.waveform.getDict()
+        #wd = self.waveform.getDict()
+        wires = self.waveform.wires
         
-        for wav in wd.keys():
-            dc_iid = tv.insert("", tkinter.END, text=wav.getFullPath(), open=True)
+        for wav in wires:
+            if (self.shortNames):
+                name = wav.name
+            else:
+                name = wav.getFullPath()
+
+            dc_iid = tv.insert("", tkinter.END, text=name, open=True)
         
         # self.map_id_obj[dc_iid] = self.sys
     
@@ -303,35 +409,42 @@ class WaveformWindow:
         self.drawWaveforms()
         self.canvas.update()
 
-        
+    def getNumClocks(self):
+        wd = self.waveform.getDict()
+        return len(list(wd.values())[0])
+    
     def drawWaveforms(self):
-        off = 45
-        vspace = 20
+        off = 43
+        vspace = 21
         wd = self.waveform.getDict()
 
         self.setColor('blue')
         
-        hclock = self.hclock
+        hclock = self.hclock # this is the width of the clock (in pixels ?)
         vsig = 15
         vtext = 7
         htrans = 3
         
-        for idx, wav in enumerate(wd.keys()):
-            if isinstance(wav, Wire):
-                w = wav.getWidth()
+        for idx, obj in enumerate(self.waveform.wires):
+            if (isinstance(obj, FieldInspector)):
+                w = obj
+                ww = -1
             else:
-                w = wav.wire.getWidth()
+                w = Waveform.getwire(obj)
+                ww = w.getWidth()
                 
-            data = wd[wav]
-            
-            if (w == 1):
+            data = wd[w]
+            fmt = self.waveform.format[idx]
+
+            print('drawing wire', idx, w.getFullPath())            
+            if (ww == 1):
                 lastval = None
             else:
                 lastval = '?' # arbitrary value 
             
             for clk in range(len(data)):
                 val = data[clk]
-                if (w == 1):
+                if (ww == 1):
                     # binary wires
                     self.drawLine(clk*hclock, off + idx*vspace - val*vsig, 
                                   (clk+1)*hclock, off + idx*vspace - val*vsig)     
@@ -353,7 +466,8 @@ class WaveformWindow:
                                   (clk+1)*hclock - htrans, off + idx*vspace - vsig)     
                         self.drawLine(clk*hclock + htrans, off + idx*vspace , 
                                   (clk+1)*hclock - htrans, off + idx*vspace )     
-                        self.drawText(clk*hclock + htrans, off + idx*vspace - vtext, hex(val), 'w')
+
+                        self.drawText(clk*hclock + htrans, off + idx*vspace - vtext, fmt.format(val), 'w')
                     else:
                         self.drawLine(clk*hclock - htrans, off + idx*vspace - vsig, 
                                   (clk+1)*hclock - htrans, off + idx*vspace - vsig)     
@@ -376,6 +490,7 @@ class WaveformWindow:
             ha = 'center'
             
         self.canvas.create_text(x,y, anchor=anchor, text=text)
+        
 @deprecated # use Waveform, just maintened for reference
 class OldWaveform(Logic):
 
@@ -514,3 +629,62 @@ class Sequence(Logic):
     def clock(self):
         self.r.prepare(self.values[self.i])
         self.i = ( self.i +1 ) % self.n
+
+
+class RandomValue(Logic):
+    def __init__(self, parent, name, r, mean, stddev):
+        super().__init__(parent, name)
+        
+        self.r = self.addOut('r', r)
+        
+        self.mean = mean
+        self.stddev = stddev
+        self.state = '{} <- {}+-{}'.format(self.r.get(), self.mean, self.stddev)
+        
+    def clock(self):
+        import numpy as np
+        v = int(np.random.normal(self.mean, self.stddev))
+        self.r.prepare(v)
+        self.state = '{} <- {}+-{}'.format(v, self.mean, self.stddev)
+        
+    def update_mean(self, v):
+        self.mean = float(v)
+        self.label1.config(text="Mean: {}".format(self.mean))
+        
+    def update_stddev(self, v):
+        self.stddev = float(v)
+        self.label2.config(text="Stddev: {}".format(self.stddev))
+        
+    def tkinter_gui(self, parent):
+        from tkinter import ttk
+        import tkinter as tk
+        frame = ttk.Frame(parent, padding="10")
+        frame.grid(column=2, row=2, sticky=(tk.W, tk.E, tk.N, tk.S))
+
+        maxv = (2 ** self.r.getWidth()) -1
+        
+        print('maxv : ', maxv)
+        
+        # Create sliders and labels
+        
+        self.label1 = ttk.Label(frame, text="Mean: {}".format(self.mean))
+        self.label2 = ttk.Label(frame, text="Stddev: {}".format(self.stddev))
+
+        slider1 = ttk.Scale(frame, from_=0, to=maxv, orient="horizontal", command=self.update_mean)
+        slider1.set(self.mean)
+        slider2 = ttk.Scale(frame, from_=0, to=maxv, orient="horizontal", command=self.update_stddev)
+        slider2.set(self.stddev)
+        
+        
+
+        # Grid layout for sliders and labels
+        slider1.grid(column=0, row=0,  pady=10)
+        slider2.grid(column=0, row=1,  pady=10)
+        self.label1.grid(column=1, row=0, padx=5)
+        self.label2.grid(column=1, row=1, padx=5)
+        
+        # Configure column weights to make the sliders and labels expandable
+        frame.columnconfigure(0, weight=1)
+        frame.columnconfigure(1, weight=1)
+        
+        return frame
\ No newline at end of file
