@@ -10,8 +10,31 @@ from .logic import *
 from .logic.bitwise import *
 from .logic.storage import *
 from .schematic_symbols import *
+from .transpilation.python2verilog_transpilation import *
 
 def getVerilogModuleName(obj:Logic, noInstanceNumber=False):
+    '''
+    Returns the module name of an object.
+    Logic classes can provide their module name by implementing the method structureName,
+    if it is not provided the structure is unique for every instance, hence the number
+    of structures is significantly increase (which is generally bad for reading)
+
+    Parameters
+    ----------
+    obj : Logic
+        DESCRIPTION.
+    noInstanceNumber : TYPE, optional
+        DESCRIPTION. The default is False.
+
+    Returns
+    -------
+    str : TYPE
+        DESCRIPTION.
+
+    '''
+    if (has_method(obj, 'structureName')):
+        return obj.structureName()
+    
     str = type(obj).__name__  
     if (not(noInstanceNumber)):
         sid = hex(id(obj))
@@ -91,10 +114,22 @@ def isReservedVerilogKeyword(str):
         return True
     return False
 
+wire_names_cache_obj = None
+wire_names_cache = None
+
+def clearWireNamesCache():
+    global wire_names_cache_obj
+    global wire_names_cache
+    wire_names_cache_obj = None
+    wire_names_cache = None
+
+
+    
 def getWireNames(obj:Logic):
     """
-    Collects the wires of the obj and their names in the
-    its scope 
+    Collects the wires of the obj and their names in 
+    its scope. Since this does not change and it could be called multiple 
+    times during the RTL generation process, we maintain a cached copy. 
 
     Parameters
     ----------
@@ -106,12 +141,22 @@ def getWireNames(obj:Logic):
         A dictionary keyed by wire objects returning their name
         in the obj scope
     """
+    global wire_names_cache_obj
+    global wire_names_cache
+
+    if (obj == wire_names_cache_obj):
+        return wire_names_cache
+
     ret = {}
     
     # process all the wires connected to child instances
     for child in obj.children.values():
         for wire in collectPortWires(child):
-            ret[wire]= "w_" + wire.name
+            if (isinstance(wire, Wire)):
+                # avoid reporting fake wires
+                ret[wire]= "w_" + wire.name
+            else:
+                ret[wire] = wire.name
  
     # overwrite the wires that are part of the interface
     for inp in obj.inPorts:
@@ -121,11 +166,18 @@ def getWireNames(obj:Logic):
     for outp in obj.outPorts:
         ret[outp.wire] = getPortName(outp) 
         
+    for outp in obj.inOutPorts:
+        ret[outp.wire] = getPortName(outp) 
+ 
+    wire_names_cache_obj = obj
+    wire_names_cache = ret
+       
     return ret
     
 def collectPortWires( obj:Logic):
     """
-    Return the wires of the object interface
+    Return the wires of the object interface.
+    @todo where is this used ?
 
     Parameters
     ----------
@@ -140,11 +192,18 @@ def collectPortWires( obj:Logic):
     ret = []
     
     for inp in obj.inPorts:
-        if (not(inp.wire is None)):
+        wire = inp.wire 
+        if (not(wire is None)):
             ret.append(inp.wire)
 
     for outp in obj.outPorts:
-        if (not(outp.wire is None)):
+        wire = outp.wire 
+        if (not(wire is None)):
+            ret.append(outp.wire)
+            
+    for outp in obj.inOutPorts:
+        wire = outp.wire
+        if (not(wire is None)):
             ret.append(outp.wire)
 
     return ret
@@ -187,9 +246,20 @@ def getPortName(p):
 
 def getWireName(scope:Logic, w:Wire):
     wNames = getWireNames(scope)
+    
+    if not(w in wNames.keys()):
+        print('ERROR: wire ', w.getFullPath(), 'not in the wires of ', scope.getFullPath()) 
+        for w2 in wNames.keys():
+            print(wNames[w2], w2 )
+        raise Exception('Wire wire {} not in the wires of {}'.format( w.getFullPath(), scope.getFullPath())) 
+        
     return wNames[w]
 
 def getParentWireName(child:Logic, w:Wire):
+    if  (isinstance(w, FakeWire)):
+        # return name for fake wires
+        return w.name
+    
     wNames = getWireNames(child.parent)
     return wNames[w]
     
@@ -199,9 +269,17 @@ def InlineConstant(obj:Logic):
 def InlineNot(obj:Logic):
     return "assign {} = ~{};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a))
 
+def InlineBidirBuf(obj:Logic):
+    str =  "assign {} = {};\n".format(getParentWireName(obj, obj.pin), getParentWireName(obj, obj.bidir))
+    str += "assign {} = ({}) ? {} : {}'bZ;\n".format(getParentWireName(obj, obj.bidir), getParentWireName(obj, obj.poe), getParentWireName(obj, obj.pout), obj.bidir.getWidth())
+    return str
+
 def InlineBuf(obj:Logic):
     return "assign {} = {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a))
 
+def InlineSignExtend(obj:Logic):
+    return "assign {} = {{ {{ {} {{ {}[{}] }} }}, {} }};\n".format(getParentWireName(obj, obj.r), obj.r.getWidth() - obj.a.getWidth(),  getParentWireName(obj, obj.a), obj.a.getWidth()-1, getParentWireName(obj, obj.a))
+
 def InlineZeroExtend(obj:Logic):
     return "assign {} = {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a))
     
@@ -223,8 +301,8 @@ def InlineXor2(obj:Logic):
 def InlineMux2(obj:Logic):
     return "assign {} = ({})? {} : {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.sel), getParentWireName(obj, obj.sel1) , getParentWireName(obj, obj.sel0))
 
-def InlineAdd(obj:Logic):
-    return "assign {} = {} + {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a) , getParentWireName(obj, obj.b))
+def InlineAddCarryIn(obj:Logic):
+    return "assign {} = {} + {} + {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a) , getParentWireName(obj, obj.b) , getParentWireName(obj, obj.ci) )
 
 def InlineMul(obj:Logic):
     return "assign {} = {} * {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a) , getParentWireName(obj, obj.b))
@@ -232,6 +310,9 @@ def InlineMul(obj:Logic):
 def InlineDiv(obj:Logic):
     return "assign {} = {} / {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a) , getParentWireName(obj, obj.b))
 
+def InlineMod(obj:Logic):
+    return "assign {} = {} % {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a) , getParentWireName(obj, obj.b))
+
 def InlineSub(obj:Logic):
     return "assign {} = {} - {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a) , getParentWireName(obj, obj.b))
 
@@ -283,12 +364,28 @@ def InlineRepeat(obj:Logic):
     return str
 
 def InlineConcatenateMSBF(obj:Logic):
-    str = ""
+    str = '' # "# MSBF \n"
     w = len(obj.ins)
     if (w == 1):
         return "assign {} = {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.ins[0]))
 
-    str = 'assign {} ='.format(getParentWireName(obj, obj.r))
+    str += 'assign {} ='.format(getParentWireName(obj, obj.r))
+    
+    link = '{'
+    for i in range(w):
+        str += link + "{}".format(getParentWireName(obj, obj.ins[i]))
+        link = ','
+    str += '};\n'
+    
+    return str
+
+def InlineConcatenateLSBF(obj:Logic):
+    str = '' # "# LSBF \n"
+    w = len(obj.ins)
+    if (w == 1):
+        return "assign {} = {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.ins[0]))
+
+    str += 'assign {} ='.format(getParentWireName(obj, obj.r))
     
     link = '{'
     for i in range(w):
@@ -333,13 +430,13 @@ def InlineVerilogCommnent(obj:Logic):
     
 def BodyReg(obj:Logic):
     clkname = getObjectClockDriver(obj).name
-    str = "reg "+getWidthInfo(obj.q) + " rq = 0;\n"
+    str = "reg "+getWidthInfo(obj.q) + " rq = {};\n".format(obj.reset_value)
     str += "always @(posedge {})\n".format(clkname)
     close = ""
     if not(obj.r is None):
         str += "if (r == 1)\n"
         str += "begin\n"
-        str += "   rq <= 0;\n"
+        str += "   rq <= {};\n".format(obj.reset_value)
         str += "end\n";
         str += "else\n";
         str += "begin\n";
@@ -369,60 +466,81 @@ def BodyGatedClock(obj:Logic):
 class VerilogGenerator:
     
     
-    def __init__(self, obj:Logic):
+    def __init__(self, obj:Logic, ast_tree=None):
         #print('Testing Verilog Generation')
         self.obj = obj
+        self.ast_tree = ast_tree
         
         self.inlinablePrimitives = {}
         
-        self.inlinablePrimitives[Add] = InlineAdd
-        self.inlinablePrimitives[Mul] = InlineMul
+        self.inlinablePrimitives[AddCarryIn] = InlineAddCarryIn
+        
         self.inlinablePrimitives[And2] = InlineAnd2
         self.inlinablePrimitives[And] = InlineAnd
-        self.inlinablePrimitives[Buf] = InlineBuf
+        self.inlinablePrimitives[BidirBuf] = InlineBidirBuf
         self.inlinablePrimitives[Bit] = InlineBit
         self.inlinablePrimitives[BitsLSBF] = InlineBitsLSBF
         self.inlinablePrimitives[BitsMSBF] = InlineBitsMSBF
+        self.inlinablePrimitives[Buf] = InlineBuf
         self.inlinablePrimitives[ConcatenateMSBF] = InlineConcatenateMSBF
+        self.inlinablePrimitives[ConcatenateLSBF] = InlineConcatenateLSBF
         self.inlinablePrimitives[Constant] = InlineConstant
         self.inlinablePrimitives[Div] = InlineDiv
         self.inlinablePrimitives[Equal] = InlineEqual
         self.inlinablePrimitives[EqualConstant] = InlineEqualConstant
+        self.inlinablePrimitives[Mod] = InlineMod
+        self.inlinablePrimitives[Mul] = InlineMul
+        self.inlinablePrimitives[Mux2] = InlineMux2
         self.inlinablePrimitives[Nand2] = InlineNand2
         self.inlinablePrimitives[Not] = InlineNot
         self.inlinablePrimitives[Nor] = InlineNor
         self.inlinablePrimitives[Nor2] = InlineNor2
         self.inlinablePrimitives[Or] = InlineOr
         self.inlinablePrimitives[Or2] = InlineOr2
-        self.inlinablePrimitives[Mux2] = InlineMux2
         self.inlinablePrimitives[Sub] = InlineSub
-        self.inlinablePrimitives[Xor2] = InlineXor2
         self.inlinablePrimitives[Range] = InlineRange
         self.inlinablePrimitives[Repeat] = InlineRepeat
+        self.inlinablePrimitives[SignExtend] = InlineSignExtend
         self.inlinablePrimitives[ZeroExtend] = InlineZeroExtend
         self.inlinablePrimitives[VerilogComment] = InlineVerilogCommnent
+        self.inlinablePrimitives[Xor2] = InlineXor2
         
         self.providingBody = {}
         
         self.providingBody[Reg] = BodyReg 
         self.providingBody[GatedClock] = BodyGatedClock
         
-    def getVerilogForHierarchy(self, obj=None, noInstanceNumberInTopEntity=True):
+        
+        
+        self.created_structures = []
+        
+    def getVerilogForHierarchy(self, obj=None, noInstanceNumberInTopEntity=True, 
+                               forceName=None, createdStructures=None):
         """
         Generates Verilog for all entities of the object hierarchy
 
         Returns
         -------
-        None.
+        An string with the Verilog for the Hierarchy.
 
         """
         
+        clearWireNamesCache()
+        
+        if (createdStructures is None):
+            self.created_structures = []
+        else:    
+            self.created_structures = createdStructures
+        
+        return self._getVerilogForHierarchy(obj, noInstanceNumberInTopEntity, forceName)
+    
+    def _getVerilogForHierarchy(self, obj=None, noInstanceNumberInTopEntity=True, forceName=None):
+        
+        
         if (obj is None):
             obj = self.obj
         
-        #print('generating {}'.format(obj.getFullPath()))
-            
-        str = self.getVerilog(obj, noInstanceNumber = noInstanceNumberInTopEntity)
+        str = self._getVerilog(obj, noInstanceNumber = noInstanceNumberInTopEntity, forceName=forceName)
         
         for child in obj.children.values():
             if (self.isInlinable(child)):
@@ -430,24 +548,63 @@ class VerilogGenerator:
                 pass
             else:
                 # skip inlinable modules from verilog generation
-                str += "\n"
-                str += self.getVerilogForHierarchy(child, noInstanceNumberInTopEntity=False)
+                part = self._getVerilogForHierarchy(child, noInstanceNumberInTopEntity=False)
+                
+                if len(part) > 0:
+                    str += "\n"
+                    str += part
 
         return str        
         
-    def getVerilog(self, obj=None, noInstanceNumber=False):
+    def getVerilog(self, obj=None, noInstanceNumber=False, forceName=None):
+        '''
+        Create Verilog for a module
+
+        Parameters
+        ----------
+        obj : TYPE, optional
+            DESCRIPTION. The default is None.
+        noInstanceNumber : TYPE, optional
+            DESCRIPTION. The default is False.
+
+        Returns
+        -------
+        str : TYPE
+            DESCRIPTION.
+
+        '''
+        clearWireNamesCache()
+        self.created_structures = []
+        return self._getVerilog(obj, noInstanceNumber, forceName)
+    
+    def _getVerilog(self, obj=None, noInstanceNumber=False, forceName=None):
+        
         str = "// This file was automatically created by py4hw RTL generator\n"
         
         if (obj is None):
             obj = self.obj
             
-            
-        str += self.createModuleHeader(obj, noInstanceNumber=noInstanceNumber)
+        # check if structure was already generated
+        
+        if not(forceName is None):
+            structure_name = forceName
+        else:
+            structure_name = getVerilogModuleName(obj, noInstanceNumber=noInstanceNumber)
+        
+        
+        if (structure_name in self.created_structures):
+            # print(f'structure {structure_name} already created in {self.created_structures}')
+            return ''
+        
+        str += self.createModuleHeader(obj, structure_name)
         
         localWires = collectLocalWires(obj)
         wireNames = getWireNames(obj)
         
         for wire in localWires:
+            if (isinstance(wire, FakeWire)):
+                continue
+            
             name = wireNames[wire]
             ww = wire.getWidth()
             if (ww > 1):
@@ -460,14 +617,19 @@ class VerilogGenerator:
             if (self.isProvidingBody(obj)):
                 str += self.provideBody(obj);
             else:
-                print('transpiling', obj.getFullPath())
+                print('transpiling combinational', obj.getFullPath())
                 str += self.generateCodeFromPropagate(obj)
 
+        elif (obj.isRunnable()):
+            print('transpiling algorithmic', obj.getFullPath())
+            str += self.generateCodeFromRun(obj)
+
         elif (obj.isClockable()):
             #generate code from clock function
             if (self.isProvidingBody(obj)):
                 str += self.provideBody(obj);
             else:
+                print('transpiling sequential', obj.getFullPath())
                 str += self.generateCodeFromClock(obj)
 
         else:
@@ -476,6 +638,8 @@ class VerilogGenerator:
         
         str += "endmodule\n"
         
+        self.created_structures.append(structure_name)
+        
         return str
         
     def anyClockableDescendant(self, obj:Logic):
@@ -501,8 +665,25 @@ class VerilogGenerator:
             
         return False
         
-    def createModuleHeader(self, obj:Logic, noInstanceNumber=None):
-        str = "module " + getVerilogModuleName(obj, noInstanceNumber=noInstanceNumber) + " (\n\t"
+    def createModuleHeader(self, obj:Logic, structureName):
+        # Created the Module Header containing inputs, ouputs and parameters
+        
+        str = "module " + structureName 
+        
+        # Add parameters
+        paramNames = obj.getParameterNames()
+        if not(paramNames is None):
+            str += " #( \n\t"
+            link = ""
+            
+            for paramName in paramNames:
+                str += link + 'parameter ' +  paramName
+                link = ',\n\t'
+                
+            str += ')\n'
+                
+            
+        str += " (\n\t"
 
         link = ""
         
@@ -513,7 +694,7 @@ class VerilogGenerator:
             str += link + "output clk_out".format(drv.name)
             
         reg = ""
-        if (obj.isClockable() and not(self.isProvidingBody(obj))):
+        if ((obj.isClockable() or obj.isPropagatable()) and not(self.isProvidingBody(obj))):
             reg = " reg "
         
         if (self.anyClockableDescendant(obj)):
@@ -528,6 +709,10 @@ class VerilogGenerator:
         for outp in obj.outPorts:
             str += link + "output " + reg + getWidthInfo(outp.wire) + " " + getPortName(outp)+ ""
             link = ",\n\t"
+
+        for outp in obj.inOutPorts:
+            str += link + "inout " + reg + getWidthInfo(outp.wire) + " " + getPortName(outp)+ ""
+            link = ",\n\t"
             
         str += ");\n"
         
@@ -550,6 +735,9 @@ class VerilogGenerator:
     def isProvidingBody(self, obj:Logic):
         # if (not(obj.isPrimitive())):
         #     return False
+
+        if (has_method(obj, 'verilogBody')):
+            return True
         
         try:
             ret = self.providingBody[type(obj)]
@@ -563,10 +751,28 @@ class VerilogGenerator:
         return ret(obj)
 
     def provideBody(self, obj:Logic):
+
+        if (has_method(obj, 'verilogBody')):
+            return obj.verilogBody()
+
         ret = self.providingBody[type(obj)]
         return ret(obj)
         
     def createModuleInstances(self, obj:Logic):
+        '''
+        Instantiate the child instances of the module
+
+        Parameters
+        ----------
+        obj : Logic
+            DESCRIPTION.
+
+        Returns
+        -------
+        str
+            The Verilog of sub-circuit instantiation.
+
+        '''
         str = "\n"
         
         for child in obj.children.values():
@@ -581,7 +787,25 @@ class VerilogGenerator:
         return str;
 
     def instantiateStructural(self, child:Logic):
-        str = getVerilogModuleName(child) + " " +  getInstanceName(child)
+        parent = child.parent
+        str = getVerilogModuleName(child) + " " 
+        
+        paramNames = child.getParameterNames()
+        if not(paramNames is None):
+            str += '#('
+            link = ''
+            
+            for paramName in paramNames:
+                paramValue = child.getParameterInstantiationValue(paramName)
+                
+                if isinstance(paramValue, Parameter):
+                    paramValue = paramValue.name
+                    
+                str += link + f'.{paramName}({paramValue})'
+                link = ','
+            str += ') '
+            
+        str += getInstanceName(child)
         str += "("
         link = ""
         
@@ -594,8 +818,19 @@ class VerilogGenerator:
             
         elif (self.anyClockableDescendant(child)):
             # Clock is an implicit parameter
-            clkname = getObjectClockDriver(child).name
-            str += link + ".{}({})".format(clkname, clkname)
+            drv:ClockDriver = getObjectClockDriver(child)
+            clkname = drv.name
+            if (drv.wire is None):
+                raise Exception('None clk driver wire for', clkname, child.getFullPath())
+                
+            parentDrv:ClockDriver = getObjectClockDriver(child.parent)
+            
+            if (drv.wire == parentDrv.wire):
+                wirename = clkname
+            else:
+                wirename = getWireName(parent, drv.wire)
+                
+            str += link + ".{}({})".format(clkname, wirename)
             link = ","
         
 
@@ -623,6 +858,17 @@ class VerilogGenerator:
  
             link = ","
         
+        for outp in child.inOutPorts:
+            if (outp.wire is None):
+                raise Exception('Input/Output port {} from {} not connected to any wire'.format(outp.name, child.getFullPath()));
+
+            if (not(outp.wire in wireName)):
+                raise Exception('Input/Output port wire {} {} not part of the wires of the parent {}'.format(outp.wire.getFullPath(), outp.wire, child.parent.getFullPath()))
+
+            str += link + "." + getPortName(outp) + "("+wireName[outp.wire]+")"
+ 
+            link = ","
+            
         str += ");\n"
 
         return str                
@@ -630,47 +876,33 @@ class VerilogGenerator:
     
     def generateCodeFromClock(self, obj:Logic):
         str = "// Code generated from clock method\n"
-        clkname = getObjectClockDriver(obj).name
         
-        tr = Python2VerilogTranspiler(obj, 'clock')
+        tr = Python2VerilogTranspiler(obj, self.ast_tree)
         
-        transpiled = tr.transpile() ;
         
-        str += "// local declarations\n"
-        str += tr.getExtraDeclarations() + "\n";
-        str += "// sequential process\n"
-        str += "always @(posedge {})\n".format(clkname)
-        str += "begin\n"
-        str += transpiled + "\n"
-        str += "end\n"
+        node = tr.transpileSequential()
+        str += tr.toVerilog(node)
         
-        str += "\n"
+        str = tr.format(str)
         
         return str
 
     def generateCodeFromPropagate(self, obj:Logic):
-        str = "// Code generated from clock method\n"
-        clkname = getObjectClockDriver(obj).name
+        str = "// Code generated from propagate method\n"
         
-        tr = Python2VerilogTranspiler(obj, 'propagate')
+        tr = Python2VerilogTranspiler(obj)
         
-        transpiled = tr.transpile() ;
+        node = tr.transpileCombinational()
+        str += tr.toVerilog(node)
         
-        str += "// local declarations\n"
-        str += tr.getExtraDeclarations() + "\n";
-        str += "// combinational process\n"
-        str += "always @(*)\n".format(clkname)
-        str += "begin\n"
-        str += transpiled + "\n"
-        str += "end\n"
-        
-        str += "\n"
+        str = tr.format(str)
         
         return str
+
 import inspect
 import ast
 import textwrap
-    
+import astunparse    
 
 def getAstValue(obj):
     if (isinstance(obj, ast.Constant)):
@@ -690,6 +922,8 @@ def getAstName(obj):
         return obj.id
     elif (isinstance(obj, ast.Attribute)):
         return getAstName(obj.attr)
+    elif (isinstance(obj, ast.Constant)):
+        return obj.value
     elif (isinstance(obj, str)):
         return obj
     elif (isinstance(obj, list)):
@@ -698,201 +932,9 @@ def getAstName(obj):
         
         return getAstName(obj[0])
     else:
-        raise Exception('unknown type {}'.format(type(obj)))
+        raise Exception('unknown type {} in {}'.format(type(obj), astunparse.unparse(obj)))
 
-class Python2VerilogTranspiler:
-
-    def __init__(self, obj:Logic, methodName:str):
-        self.obj = obj
-        self.methodName = methodName
-        self.signals = {}
-        self.indent = 0
-
-    def getIndent(self):
-        return ' ' * (self.indent * 4)
-        
-    def transpile(self):
-        
-        methods = inspect.getmembers(self.obj, inspect.ismethod)
-        method = [x[1] for x in methods if x[0] == self.methodName ][0]
-
-        source = textwrap.dedent(inspect.getsource(method))
-        module = ast.parse(source)
-
-        
-        module = ReplaceWireGets().visit(module)
-        module = ReplaceWirePrepare().visit(module)
-        module = ReplaceWirePut().visit(module)
-        
-        func = module.body[0].body
-        
-        return self.transpileUnknown(func)
-        
-    def getExtraDeclarations(self):
-        str = ""
-        
-        portNames = []
-        
-        
-        for inp in self.obj.inPorts:
-            portNames.append(getValidVerilogName(inp.name))
-
-        for outp in self.obj.outPorts:
-            portNames.append(getValidVerilogName(outp.name))        
-        
-        #print('SIGNALS:', self.signals)
-        #print('PORT NAMES:', portNames)
-
-        extra = [x for x in self.signals if x not in portNames]
-        
-        # @todo we should analyze the number of possible values of 
-        # extra signals to decide their width. By now we consider a worst
-        # case scenario with extra signals all requiring a maximum of 8 bits
-        # this will be simplyfied during synthesis if less bits are required
-        # but it will cause a BUG if the required bits are higher
-        for sig in extra:
-            str += "reg [7:0] " + sig + " = 0;\n" 
-            
-        return str
-    
-    def transpileUnknown(self, line):
-        
-        if (type(line) == ast.If):
-            return self.transpileIf(line);
-        if (type(line) == ast.Compare):
-            return self.transpileCompare(line)
-        if (type(line) == ast.Assign):
-            return self.transpileAssign(line)
-        if (type(line) == ast.BinOp):
-            return self.transpileBinOp(line)
-        if (type(line) == ast.Attribute):
-            return self.transpileAttribute(line)
-        if (isinstance(line, ast.Name)):
-            return self.transpileName(line)
-        if (type(line) == ast.Eq):
-            return "==";
-        if (type(line) == ast.Constant):
-            return "{}".format(getAstValue(line))
-        if (type(line) == ast.Num):
-            return "{}".format(int(line.n))
-        
-        if (type(line) == list):
-            str = ""
-            for item in line:
-                str += self.transpileUnknown(item) 
-            return str
-
-        if (type(line) == ast.Expr):
-            return self.transpileExpr(line)
-        
-        else:
-            str= "type = {}\n".format(type(line))
-            
-            str += ast.dump(line) + "\n"
-            
-            return str
-        
-    def transpileAttribute(self, line:ast.Attribute):
-        return line.attr
-            
-    def transpileExpr(self, line:ast.Expr):
-        return self.transpileUnknown(line.value)
-        
-    def transpileBinOp(self, line:ast.BinOp):
-        return "(" + self.transpileUnknown(line.left) + self.transpileOp(line.op) + self.transpileUnknown(line.right) + ")";
-
-    def transpileOp(self, op):
-        if (isinstance(op, ast.BitAnd)):
-            return '&'
-        elif (isinstance(op, ast.BitOr)):
-            return '|'
-        elif (isinstance(op, ast.BitXor)):
-            return '^'
-        else:
-            raise Exception('Op not supported: {}'.format(op))
-        
-    def transpileAssign(self, line:ast.Assign):
-        targets = line.targets 
-        if (len(targets) > 1):
-            return ast.dump(line)
-        
-        var = getAstName(targets[0])
-        
-        self.signals[var] = var
-        
-        return var + " <= " + self.transpileUnknown(line.value) + ";\n" + self.getIndent()  ;
-    
-    def transpileIf(self, line:ast.If):
-        str = "if " + self.transpileUnknown(line.test) + "\n"
-        str += self.getIndent() + "begin\n"
-        self.indent += 1
-        str += self.getIndent() + self.transpileUnknown(line.body)
-        self.indent -= 1
-        str +=  "end\n"
-        
-        if (not(line.orelse is None)):
-            str += self.getIndent() + "else \n"
-            str += self.getIndent() + "begin\n"
-            self.indent += 1
-            str += self.getIndent() + self.transpileUnknown(line.orelse)
-            self.indent -= 1
-            str +=  "end\n"
-
-        return str
-    
-    def transpileCompare(self, line:ast.Compare):
-        str = "("
-        str += self.transpileUnknown(line.left)
-        str += self.transpileUnknown(line.ops)
-        str += self.transpileUnknown(line.comparators)
-        str += ")"
-        return str;
-    
-    def transpileName(self, line:ast.Name):
-        str =  getValidVerilogName(line.id)
-        self.signals[str] = str
-        return str
-    
-    
-    
-
-class ReplaceWireGets(ast.NodeTransformer):
-    def visit_Call(self, node):
-        attr = getAstName(node.func)
-        
-        if (attr == 'get'):
-            #if isinstance(node.func.value, ast.Attribute):
-            wirename = getAstName(node.func.value)
-            #print('REPLACING GET ', wirename)
-            return ast.Name(wirename, ast.Load)
-        
-        return node
-    
-    
-class ReplaceWirePrepare(ast.NodeTransformer):
-    def visit_Call(self, node):
-        attr = getAstName(node.func)
-        
-        if (attr == 'prepare'):
-            #print('REPLACE WIRE PUTS FUNC:', attr , node.func.value.attr, node.args)
-            return ast.Assign([node.func.value], node.args[0])
-        
-        
-        return node
-    
-class ReplaceWirePut(ast.NodeTransformer):
-    def visit_Call(self, node):
-        attr = getAstName(node.func)
-        
-        if (attr == 'put'):
-            #print('REPLACE WIRE PUTS FUNC:', attr , node.func.value.attr, node.args)
-            return ast.Assign([node.func.value], node.args[0])
-        
-        
-        return node
-    
-    
 class VerilogComment(Logic):
     def __init__(self, parent, name: str, comment:str):
         super().__init__(parent, name)
-        self.comment = comment
\ No newline at end of file
+        self.comment = comment
