@@ -22,6 +22,7 @@ cellmargin_initial = 150
 netmargin = 40
 netspacing = 10
 nettrackspacing = 10
+portvaluemargin = 10
 
 class MatplotlibRender:
     def __init__(self, shape, physical_shape=None, dpi=None):
@@ -127,6 +128,11 @@ class MatplotlibRender:
         xnew,ynew = interpolate.splev( np.linspace( 0, 1, 20 ), tck,der = 0)
         self.canvas.plot( xnew ,ynew , color=self.color, linewidth=self.linewidth)
         
+    def drawImage(self, x, y, img ):
+        w = img.shape[1]
+        h = img.shape[0]
+        self.canvas.imshow(img, extent=[x,x+w,y+h,y], origin='upper')
+        
 
 class TkinterRender:
     # check https://zetcode.com/tkinter/drawing/
@@ -134,6 +140,13 @@ class TkinterRender:
     def __init__(self, parent, shape):
         from tkinter import Canvas
         self.canvas = Canvas(parent, bg='white')
+        
+        self.canvas.bind("<ButtonPress-1>", self.on_press)
+        self.canvas.bind("<B1-Motion>", self.on_drag)
+
+        self.start_x = None
+        self.start_y = None
+        
         w = shape[0]
         h = shape[1]
         dpi = 100
@@ -153,6 +166,25 @@ class TkinterRender:
         self.linewidth = 2
         self.xmargin = 50
         self.ymargin = 50
+
+    def on_press(self, event):
+        self.start_x = int(self.canvas.canvasx(event.x))
+        self.start_y = int(self.canvas.canvasy(event.y))
+
+    def on_drag(self, event):
+        if self.start_x is not None and self.start_y is not None:
+            cur_x = int(self.canvas.canvasx(event.x))
+            cur_y = int(self.canvas.canvasy(event.y))
+
+            delta_x = cur_x - self.start_x
+            delta_y = cur_y - self.start_y
+
+            self.canvas.scan_mark(self.start_x, self.start_y)
+            self.canvas.scan_dragto(cur_x, cur_y, gain=1)
+
+    def reset_start_coords(self):
+        self.start_x = None
+        self.start_y = None
         
     def setForecolor(self, color):
         #print('color {} = {}'.format(color, colors.rgb2hex(colors.to_rgb(color))))
@@ -235,6 +267,19 @@ class TkinterRender:
         xnew,ynew = interpolate.splev( np.linspace( 0, 1, 20 ), tck,der = 0)
         self.drawPolygon( xnew ,ynew)
 
+    def drawImage(self, x, y, np_img ):
+        from tkinter import PhotoImage
+        from PIL import Image, ImageTk
+        w = np_img.shape[1]
+        h = np_img.shape[0]
+        
+        #print('drawImage', np_img.shape, type(np_img), type(np_img[0][0][0]))
+
+        pil_img = Image.fromarray(np_img)
+        tk_img = ImageTk.PhotoImage(image=pil_img)
+        # Display the image on the canvas
+        self.canvas.create_image(x, y, anchor="nw", image=tk_img)
+        self.canvas.image = tk_img
         
 class Schematic:
     """
@@ -242,8 +287,10 @@ class Schematic:
     """
 
     step_timeout = 10 # any step of the schematic rendering cannot take more than this value (seconds)
+
+    mapping = {} # maaping from object class to symbol
     
-    def __init__(self, obj:Logic, render='matplotlib', parent=None, placeAndRoute=True):
+    def __init__(self, obj:Logic, render='matplotlib', parent=None, placeAndRoute=True, showValues=False):
    
         if not(obj.isStructural()):
             raise Exception('Schematics are only available to structural circuits')
@@ -269,28 +316,27 @@ class Schematic:
 
         self.objs = []      # a list of the logic symbols displayed
         self.nets = []      # a list of the nets
-        self.sources = []   # a list of net sources with tuples [symbol, x, y, wire]
-        self.sinks = []     # a list of net sinks with tuples   [symbol, x, y, wire]
+        self.sources = []   # a list of net sources with tuples [symbol, x, y, port]
+        self.sinks = []     # a list of net sinks with tuples   [symbol, x, y, port]
         
-        self.mapping = {}
-        self.mapping[And2] = AndSymbol
-        self.mapping[And] = AndSymbol
-        self.mapping[Not] = NotSymbol
-        self.mapping[Or2] = OrSymbol
-        self.mapping[Or] = OrSymbol
-        self.mapping[Nor2] = NorSymbol
-        self.mapping[Xor2] = XorSymbol
+        Schematic.mapping[And2] = AndSymbol
+        Schematic.mapping[And] = AndSymbol
+        Schematic.mapping[Not] = NotSymbol
+        Schematic.mapping[Or2] = OrSymbol
+        Schematic.mapping[Or] = OrSymbol
+        Schematic.mapping[Nor2] = NorSymbol
+        Schematic.mapping[Xor2] = XorSymbol
         
-        self.mapping[Add] = AddSymbol
-        self.mapping[Sub] = SubSymbol
-        self.mapping[Mul] = MulSymbol
+        Schematic.mapping[Add] = AddSymbol
+        Schematic.mapping[Sub] = SubSymbol
+        Schematic.mapping[Mul] = MulSymbol
         
-        self.mapping[Reg] = RegSymbol
-        self.mapping[Scope] = ScopeSymbol
-        self.mapping[Buf] = BufSymbol
-        self.mapping[Bit] = BitSymbol
-        self.mapping[Mux2] = Mux2Symbol
-        self.mapping[Range] = RangeSymbol
+        Schematic.mapping[Reg] = RegSymbol
+        Schematic.mapping[Scope] = ScopeSymbol
+        Schematic.mapping[Buf] = BufSymbol
+        Schematic.mapping[Bit] = BitSymbol
+        Schematic.mapping[Mux2] = Mux2Symbol
+        Schematic.mapping[Range] = RangeSymbol
 
         self.mapping[Waveform] = ScopeSymbol # Temp solution
 
@@ -298,6 +344,8 @@ class Schematic:
         self.render = render        
         self.canvas = None
 
+        self.showValues = showValues
+
         if (placeAndRoute):
             self.placeAndRoute()
         
@@ -309,34 +357,39 @@ class Schematic:
         
         #mainloop()
 
-    def placeAndRoute(self):
+    def placeAndRoute(self, debug=False):
         self.placeInputPorts()
         self.placeInstances()
         self.placeOutputPorts()
 
-        self.bruteForceSort()
-        self.columnAssignment()
-        
-        self.createNets()                   
-        self.passthroughCreation()
-        self.removeArrowsSpecialCases()
-        
-        self.rowAssignment()
-        
-
-        self.trackAssignment()
-        self.replaceAsColRow()
-        
-        #self.replaceByAdjacencyMatrix()
-        
-        # self.replaceByDependency()
-        
-        #self.replaceVerticalCompress()
-        # #self.replaceHorizontalCompress()
-        
-
-        self.routeNets()
-        
+        try:
+            self.bruteForceSort()
+            self.columnAssignment()
+            
+            self.createNets(debug=debug) 
+            self.passthroughCreation()
+            self.removeArrowsSpecialCases()
+        except Exception as e:
+            print(e)
+            
+        try:
+            self.rowAssignment()
+            
+    
+            self.trackAssignment()
+            self.replaceAsColRow()
+            
+            #self.replaceByAdjacencyMatrix()
+            
+            # self.replaceByDependency()
+            
+            #self.replaceVerticalCompress()
+            # #self.replaceHorizontalCompress()
+            
+    
+            self.routeNets()
+        except Exception as e:
+            print(e)
         
     def draw(self):
         self.drawAll()
@@ -432,7 +485,7 @@ class Schematic:
             wire = inp.wire
             
             for src in self.sources:
-                if (src['wire'] == wire):
+                if (src['port'].wire == wire):
                     ret.append(src['symbol'])
         
         return ret
@@ -464,6 +517,8 @@ class Schematic:
             # an output port cannot drive anything (at the circuit level)
             # so, no dependent instances 
             return []
+        elif (isinstance(obj, InOutPort)):
+            outports = [obj]
         else:
             if (not(hasattr(obj, 'outPorts'))):
                 raise Exception('obj {} from type {} {} has not out ports'.format(obj.getFullPath(), type(obj), isinstance(obj, InPort)))
@@ -476,7 +531,7 @@ class Schematic:
             if not(wire in self.maxfanoutwires):
                 # avoid to report elements connected by pruned wires
                 for src in self.sinks:
-                    if (src['wire'] == wire):
+                    if (src['port'].wire == wire):
                         ret.append(src['symbol'])
         
         # if multiple wires connect between two symbols, there will be repetitions
@@ -649,6 +704,7 @@ class Schematic:
 
         for colidx, col in enumerate(self.columns):
             for obj in col:
+                # We list the objects in the column 
                 if (isinstance(obj, PassthroughSymbol)):
                     continue
                 if (isinstance(obj, FeedbackStartSymbol)):
@@ -670,7 +726,8 @@ class Schematic:
 
     def insertPassthrough(self, source:LogicSymbol, sourcecol:int, sink:LogicSymbol, sinkcol:int, debug=False):
         """
-        A passthrough instance has to be inserted for every wire connecting both entities
+        A passthrough instance has to be inserted for every wire connecting 
+        entities source and sink
 
         Parameters
         ----------
@@ -696,6 +753,9 @@ class Schematic:
               
         
         for wire in intersection:
+            if (wire is None):
+                continue
+            
             # remove the original net
             removeNets = [x for x in self.nets if x.wire== wire and x.source == source and x.sink == sink]
 
@@ -703,28 +763,39 @@ class Schematic:
                 self.dumpNets()
                 raise Exception('Wire:{} with source:{} {} and sink:{} {} not in remove nets'.format(wire.getFullPath(), id(source), source.obj.getFullPath(), id(sink), sink.obj.getFullPath()))
             
-            self.nets.remove(removeNets[0])
+            if (len(removeNets) > 1):
+                self.dumpNets()
+                raise Exception('Muliple nets between source:{} {} and sink:{} {}'.format( type(source).__name__, source.obj.getFullPath(), type(sink).__name__, sink.obj.getFullPath()))
+
+            netToRemove = removeNets[0]
+            self.nets.remove(netToRemove)
         
             lastSymbol = source
+            lastSourcePort = netToRemove.sourcePort
             
             for col in range(sourcecol+1, sinkcol):
                 pts = PassthroughSymbol()
-                pts.debug = debug
+
                 self.objs.append(pts)
                 self.columns[col].append(pts)
-                self.sources.append({'symbol':pts, 'wire':wire})
-                self.sinks.append({'symbol':pts, 'wire':wire})
+
+                # raise Exception('TODO get the port of this wire')
+                # self.sources.append({'symbol':pts, 'port':wire})
+                # self.sinks.append({'symbol':pts, 'port':wire})
                 
-                net1 = NetSymbol(wire, lastSymbol, pts)
+                # TODO what to do here with the port info ??
+                net1 = NetSymbol(wire, lastSourcePort, None, lastSymbol, pts)
                 net1.sourcecol = col-1
                 net1.arrow = False
                 self.nets.append(net1)
                 
                 lastSymbol = pts
+                lastSourcePort = None
                 
                 
             if (lastSymbol != None):
-                net2 = NetSymbol(wire, pts, sink)
+                # TODO what to do here with the port info 
+                net2 = NetSymbol(wire, None, netToRemove.sinkPort, pts, sink)
                 net2.sourcecol = sinkcol-1
                 self.nets.append(net2)
                                 
@@ -734,7 +805,7 @@ class Schematic:
             
     def insertFeedback(self, source:LogicSymbol, sourcecol:int, sink:LogicSymbol, sinkcol:int, debug=False):
         """
-        A passthrough instance has to be inserted for every wire connecting both entities
+        Create feedback nets between source and sink
 
         Parameters
         ----------
@@ -758,7 +829,8 @@ class Schematic:
         sink_wires = self.getWiresFromSink(sink)
         intersection = Intersection(source_wires, sink_wires)
         
-       
+        # intersection wires are wires that connect source and sink
+        # but take care, because the same wire can have multiple sink ports
         
         for wire in intersection:
             # remove the original net
@@ -768,7 +840,12 @@ class Schematic:
                 self.dumpNets()
                 raise Exception('Wire:{} with source:{} {} and sink:{} {} not in remove nets'.format(wire.getFullPath(), type(source).__name__, source.obj.getFullPath(), type(sink).__name__, sink.obj.getFullPath()))
 
-            self.nets.remove(removeNets[0])
+            if (len(removeNets) > 1):
+                self.dumpNets()
+                raise Exception('Muliple nets between source:{} {} and sink:{} {}'.format( type(source).__name__, source.obj.getFullPath(), type(sink).__name__, sink.obj.getFullPath()))
+
+            netToRemove = removeNets[0]
+            self.nets.remove(netToRemove)
         
             lastSymbol = source
             
@@ -782,7 +859,8 @@ class Schematic:
             #self.sources.append({'symbol':fb_start, 'wire':wire})
             #self.sinks.append({'symbol':fb_start, 'wire':wire})
             
-            net1 = NetSymbol(wire, lastSymbol, fb_start)
+            # TODO: what to do here with the port info ???
+            net1 = NetSymbol(wire, netToRemove.sourcePort, None , lastSymbol, fb_start)
             net1.sourcecol = sourcecol
             net1.arrow = False
             self.nets.append(net1)            
@@ -794,7 +872,8 @@ class Schematic:
             #self.sources.append({'symbol':fb_start, 'wire':wire})
             #self.sinks.append({'symbol':fb_start, 'wire':wire})
             
-            net2 = NetSymbol(wire, fb_end, sink)
+            # TODO what to do here with the port info ????
+            net2 = NetSymbol(wire, None, netToRemove.sinkPort, fb_end, sink)
             net2.sourcecol = sinkcol
             net2.arrow = True
             self.nets.append(net2)
@@ -803,32 +882,35 @@ class Schematic:
             
             
             for col in range(sinkcol+1, sourcecol):
-                  pts = PassthroughSymbol()
-                  pts.debug = debug
-                  self.objs.append(pts)
-                  self.columns[col].append(pts)
-                  self.sources.append({'symbol':pts, 'wire':wire})
-                  self.sinks.append({'symbol':pts, 'wire':wire})
-                
-                  net1 = NetSymbol(wire, lastSymbol, pts)
-                  net1.sourcecol = col-1
-                  net1.arrow = False
-                  self.nets.append(net1)
-                
-                  lastSymbol = pts
+                pts = PassthroughSymbol()
+                self.objs.append(pts)
+                self.columns[col].append(pts)
+
+                #raise Exception('TODO get the port of this wire')
+                #self.sources.append({'symbol':pts, 'port':wire})
+                #self.sinks.append({'symbol':pts, 'port':wire})
+              
+                # TODO what to do here with the port info ???
+                net1 = NetSymbol(wire, None, None, lastSymbol, pts)
+                net1.sourcecol = col-1
+                net1.arrow = False
+                self.nets.append(net1)
+              
+                lastSymbol = pts
                 
                 
             if (lastSymbol != None):
-                 net2 = NetSymbol(wire, lastSymbol, fb_start)
-                 net2.sourcecol = sourcecol
-                 net2.arrow = False
-                 self.nets.append(net2)
+                # TODO What to do here with the port info ???
+                net2 = NetSymbol(wire, None, None, lastSymbol, fb_start)
+                net2.sourcecol = sourcecol
+                net2.arrow = False
+                self.nets.append(net2)
 
     def getWiresFromSource(self, source):
-        return [x['wire'] for x in self.sources if x['symbol'] == source]
+        return [x['port'].wire for x in self.sources if x['symbol'] == source]
 
     def getWiresFromSink(self, sink):
-        return [x['wire'] for x in self.sinks if x['symbol'] == sink]
+        return [x['port'].wire for x in self.sinks if x['symbol'] == sink]
     
     def replaceByAdjacencyMatrix(self):
         am = self.getAdjacencyMatrix()
@@ -949,7 +1031,7 @@ class Schematic:
             iterNum = iterNum + 1
             changed = False
             for sourceTuple in self.sources:
-                sinks = self.findSinkTuples(sourceTuple['wire'])
+                sinks = self.findSinkTuples(sourceTuple['port'].wire)
                 sourceObj = sourceTuple['symbol']
                 
                 
@@ -978,8 +1060,10 @@ class Schematic:
                 
         
     def countNetsBetweenSymbols(self, sourceSym, sinkSym):
-        wiresFromSource = [source['wire'] for source in self.sources if source['symbol'] == sourceSym]
-        wiresToSink = [sink['wire'] for sink in self.sinks if sink['symbol'] == sinkSym]
+        raise Exception('This counts wires, not ports')
+
+        wiresFromSource = [source['port'].wire for source in self.sources if source['symbol'] == sourceSym]
+        wiresToSink = [sink['port'].wire for sink in self.sinks if sink['symbol'] == sinkSym]
         common = [wire for wire in wiresFromSource if wire in wiresToSink ]
         return len(common)
     
@@ -998,8 +1082,8 @@ class Schematic:
         for inp in self.sys.inPorts:
             isym = InPortSymbol(inp, self.x, self.y)
             self.objs.append(isym)
-            self.y = self.y + gridsize * portSeparation
-            self.sources.append({'symbol':isym, 'x':15, 'y':8+5, 'wire':inp.wire})
+            self.y = self.y + gridsize * LogicSymbol.portSeparation
+            self.sources.append({'symbol':isym, 'x':15, 'y':8+5, 'port':inp})
         
         if (len(self.sys.inPorts) > 0):
             self.x = self.x + 25 * gridsize
@@ -1013,7 +1097,7 @@ class Schematic:
         
     def getSymbol(self, obj, x, y):
         try:
-            ret = self.mapping[type(obj)]
+            ret = Schematic.mapping[type(obj)]
     
             #print('getSymbol -> good', obj)
                 
@@ -1036,12 +1120,12 @@ class Schematic:
         i = 0
         for inp in child.inPorts:
             #print('adding inport ', child, inp.name)
-            self.sinks.append({'symbol':isym, 'x':0, 'y':8+8+8+5+i*portpitch, 'wire':inp.wire})
+            self.sinks.append({'symbol':isym, 'x':0, 'y':8+8+8+5+i*LogicSymbol.portpitch, 'port':inp})
             i = i+1
         i = 0
         for inp in child.outPorts:
             #print('adding outport ', child, inp.name)
-            self.sources.append({'symbol':isym, 'x':isym.getWidth(), 'y':8+8+8+5+i*portpitch, 'wire':inp.wire})
+            self.sources.append({'symbol':isym, 'x':isym.getWidth(), 'y':8+8+8+5+i*LogicSymbol.portpitch, 'port':inp})
             i = i+1
                 
         return isym
@@ -1051,16 +1135,28 @@ class Schematic:
         self.y = gridsize*3
         
         #print('placeInstances', self.sys)
+        if (self.showValues):
+            self.x += portvaluemargin
         
         for child in self.sys.children.values():
             isym = self.placeInstance(child)
             maxx = max(maxx, isym.getWidth())
             
-        self.x = self.x + maxx + gridsize*10 + cellmargin
-        self.grid_yunits = self.y / gridsize
+        self.x += maxx + gridsize*10 + cellmargin + portvaluemargin
         self.grid_yunits = self.y / gridsize
+        self.grid_yunits = self.y / gridsize # @todo should this be x ??
         
     def placeOutputPorts(self):
+        '''
+        Place Output Ports and InOutPorts.
+        This basically means creating an output port symbol for each port and
+        assigning the its y coordinate 
+
+        Returns
+        -------
+        None.
+
+        '''
         #self.x = 1
         self.y = gridsize * 5
         
@@ -1069,9 +1165,16 @@ class Schematic:
         for inp in self.sys.outPorts:
             osym = OutPortSymbol(inp, self.x, self.y)
             self.objs.append(osym)
-            self.y = self.y + gridsize * portSeparation
-            self.sinks.append({'symbol':osym, 'x':0, 'y':8+5, 'wire':inp.wire})
-        
+            self.y = self.y + gridsize * LogicSymbol.portSeparation
+            self.sinks.append({'symbol':osym, 'x':0, 'y':8+5, 'port':inp})
+
+        for inp in self.sys.inOutPorts:
+            osym = InOutPortSymbol(inp, self.x, self.y)
+            self.objs.append(osym)
+            self.y = self.y + gridsize * LogicSymbol.portSeparation
+            self.sinks.append({'symbol':osym, 'x':0, 'y':8+5, 'port':inp})
+            self.sources.append({'symbol':osym, 'x':0, 'y':8+5, 'port':inp})
+
         #self.x = self.x + 3
 
         self.lastOutput = len(self.objs)
@@ -1079,20 +1182,20 @@ class Schematic:
         
     def findSourceTuple(self, sinkWire:Wire):
         for source in self.sources:
-            if (source['wire'] == sinkWire):
+            if (source['port'].wire == sinkWire):
                 return source
             
         sourceNames = [x['wire'].getFullPath() for x in self.sources]
-        raise Exception('No source to wire "{}" in {}'.format(sinkWire.getFullPath(), sourceNames ) )
+        raise Exception('Could not find a source to wire "{}" in the sources collection {}'.format(sinkWire.getFullPath(), sourceNames ) )
             
     def findSinkTuples(self, sourceWire):
         ret = []
         for sink in self.sinks:
-            if (sink['wire'] == sourceWire):
+            if (sink['port'].wire == sourceWire):
                 ret.append(sink)
         return ret;
             
-    def createNets(self):
+    def createNets(self, debug=False):
         """
         Create nets for all entities in the drawing.
         It populates the self.nets list by analyzing the sinks
@@ -1105,12 +1208,15 @@ class Schematic:
         """
         for sink in self.sinks:
             try:
-                wire = sink['wire']
+                if (debug):
+                    print('Sink:', sink)
+                    
+                wire = sink['port'].wire
                 source = self.findSourceTuple(wire)
                 sourcecol = self.getSymbolColumn(source['symbol'])
                 sinkcol = self.getSymbolColumn(sink['symbol'])
 
-                net = NetSymbol(wire, source['symbol'], sink['symbol'])
+                net = NetSymbol(wire, source['port'], sink['port'], source['symbol'], sink['symbol'])
                 net.sourcecol = sourcecol
                 net.sinkcol = sinkcol
                 self.nets.append(net)   
@@ -1172,7 +1278,7 @@ class Schematic:
         """
         sourceWires = {}
         for src in self.sinks:
-            wire = src['wire']
+            wire = src['port'].wire
             try:
                 symlist = sourceWires[wire]
             except:
@@ -1184,13 +1290,13 @@ class Schematic:
             
         for sink in self.sinks:
             try:
-                wire = sink['wire']
+                wire = sink['port'].wire
                 
                 fanout = len(sourceWires[wire])
                 
                 if (fanout <= maxfanout):
                     source = self.findSourceTuple(wire)
-                    self.nets.append(NetSymbol(wire, source['symbol'], sink['symbol']))   
+                    self.nets.append(NetSymbol(wire, source['port'], sink['port'], source['symbol'], sink['symbol']))   
                 else:
                     print('skip wire {} fanout: {}'.format(wire.getFullPath(), fanout))
                     
@@ -1238,7 +1344,28 @@ class Schematic:
         else:
             raise Exception('Unsupported render {}'.format(render))
 
-    def drawAll(self):
+    def drawValues(self, symbol):
+        if (symbol.obj is None):
+            #print('WARNING: no symbol for object', type(symbol))
+            return
+        if not(hasattr(symbol.obj, 'inPorts')):
+            return
+        
+        for port in symbol.obj.inPorts:
+            wire = port.wire
+            pos = symbol.getPortSinkPos(port)
+            self.canvas.drawText(symbol.x + pos[0] - portvaluemargin,
+                                 symbol.y + pos[1] - gridsize * 2,
+                                 '{:X}'.format(wire.get()), 'w')
+        for port in symbol.obj.outPorts:
+            wire = port.wire
+            pos = symbol.getPortSourcePos(port)
+            self.canvas.drawText(symbol.x + pos[0] + portvaluemargin,
+                                 symbol.y + pos[1] - gridsize * 2,
+                                 '{:X}'.format(wire.get()), 'e')
+
+
+    def drawAll(self, debug=False):
         if (self.canvas is None):
             self.createRender()
             
@@ -1252,7 +1379,13 @@ class Schematic:
             self.canvas.setForecolor('k')  
             self.canvas.setFillcolor('k')
             self.canvas.setLineWidth(2)
-            obj.draw(self.canvas)
+            if (debug):
+                print('draw', type(obj))
+            
+            obj.draw(self.canvas, debug=debug)
+            
+            if (self.showValues):
+                self.drawValues(obj)
 
         # Draw Nets
         for obj in self.getNets():
@@ -1260,8 +1393,11 @@ class Schematic:
             self.canvas.setForecolor(obj.color)  
             self.canvas.setFillcolor(obj.color)
             self.canvas.setLineWidth(1)
+
+            if (debug):
+                print('draw', type(obj))
             
-            obj.draw(self.canvas)
+            obj.draw(self.canvas, debug=debug)
         
         return self.canvas
     
@@ -1366,6 +1502,8 @@ class Schematic:
             outwires = [srcobj.obj.wire]
         elif (isinstance(srcobj, OutPortSymbol)):
             outwires = []
+        elif (isinstance(srcobj, InOutPortSymbol)):
+            outwires = [srcobj.obj.wire] # @todo is this what we should do ?
         else:
             outwires = [outport.wire for outport in srcobj.obj.outPorts]
         
@@ -1373,6 +1511,8 @@ class Schematic:
             inwires = []
         elif (isinstance(trgobj, OutPortSymbol)):
             inwires = [trgobj.obj.wire]
+        elif (isinstance(trgobj, InOutPortSymbol)):
+            inwires = [trgobj.obj.wire] # @todo check this, not sure if we should put more wires here
         else:
             inwires = [inport.wire for inport in trgobj.obj.inPorts]
         
@@ -1413,7 +1553,8 @@ class Schematic:
         if (mode == 'prerouting'):
             maxh = maxh*2
             
-        grid = np.zeros((maxh,maxw))
+        #print('ocupancy grid = ', maxh, maxw)
+        grid = np.zeros((maxh, maxw))
         
         for obj in self.getNonNets():
             if (obj in discard):
