@@ -8,16 +8,7 @@ from .logic.bitwise import *
 from .base import Wire
 import math
 
-gridsize = 5
-portpitch = 28
-cellmargin = 50
-portSeparation = 10
 
-namemargin = 8
-portmargin = 8
-instanceportwidth = 4
-instanceportheight = 10
-instanceporttextmargin = 10
  
 #
 #  name              name margin
@@ -30,13 +21,71 @@ instanceporttextmargin = 10
 #  +------------+
 
 class LogicSymbol:
+    gridsize = 5
+    portpitch = 28
+    cellmargin = 50
+    portSeparation = 10
+
+    namemargin = 8
+    portmargin = 8
+    instanceportwidth = 4
+    instanceportheight = 10
+    instanceporttextmargin = 10
+
     def __init__(self, obj:Logic, x:int, y:int):
         self.obj = obj
         self.x = x
         self.y = y
+        self.instanceWidth = self.computeWidth() 
+        #print('instance width:', self.instanceWidth)
+        
+    def getTextExtend(self, text):
+        from matplotlib.textpath import TextPath
+
+        fontsize = 12
+        
+        path = TextPath((0, 0), text, size=fontsize)
+        
+        text_width = path.get_extents().width
+        #text_height = path.get_extents().height
+        return text_width
+
+    def getInPortsWidth(self):
+        inMaxWidth = 0
+        if (hasattr(self.obj, 'inPorts')):
+            for idx, port in enumerate(self.obj.inPorts):
+                inMaxWidth = max(inMaxWidth, self.getTextExtend(port.name))
+        return inMaxWidth
+    
+    def getOutPortsWidth(self):
+        outMaxWidth = 0
+        if (hasattr(self.obj, 'outPorts')):
+            for idx, port in enumerate(self.obj.outPorts):
+                outMaxWidth = max(outMaxWidth, self.getTextExtend(port.name))
+        return outMaxWidth
+                                
+    
+    def computeWidth(self):
+        outMaxWidth = self.getOutPortsWidth()
+        inMaxWidth = self.getInPortsWidth()
+        
+        return int(outMaxWidth * 1.5  +  LogicSymbol.gridsize*10 + inMaxWidth * 1.5 )
+
+    def getPortSourcePos(self, refport):
+        selidx = -1
+        for idx, port in enumerate(self.obj.outPorts):
+            if (port.name == refport.name):
+                selidx = idx
                 
+        if (selidx == -1):
+            raise Exception('out port {} not found in {}'.format(refport.name, self.obj.getFullPath()) )
+
+
+        return (self.getWidth(), LogicSymbol.namemargin + LogicSymbol.portmargin + selidx*LogicSymbol.portpitch + LogicSymbol.instanceportheight//2)
         
     def getWireSourcePos(self, wire:Wire):
+        # TODO remove
+        raise Exception('use port one')
         selidx = -1
         for idx, port in enumerate(self.obj.outPorts):
             if (port.wire == wire):
@@ -46,9 +95,23 @@ class LogicSymbol:
             raise Exception('out port not found in {}'.format(self.obj.getFullPath()) )
 
 
-        return (self.getWidth(), namemargin + portmargin + selidx*portpitch + instanceportheight//2)
+        return (self.getWidth(), LogicSymbol.namemargin + LogicSymbol.portmargin + selidx*LogicSymbol.portpitch + LogicSymbol.instanceportheight//2)
+
+    def getPortSinkPos(self, portref):
+        selidx = -1
+        for idx, port in enumerate(self.obj.inPorts):
+            if (port == portref):
+                selidx = idx
+                
+        if (selidx == -1):
+            raise Exception('in port not found in {}'.format(self.obj.getFullPath()) )
+
+        
+        return (0, LogicSymbol.namemargin + LogicSymbol.portmargin + selidx*LogicSymbol.portpitch + LogicSymbol.instanceportheight//2)
     
     def getWireSinkPos(self, wire:Wire):
+        # TODO remove
+        raise Exception('use port one')
         selidx = -1
         for idx, port in enumerate(self.obj.inPorts):
             if (port.wire == wire):
@@ -58,13 +121,13 @@ class LogicSymbol:
             raise Exception('in port not found in {}'.format(self.obj.getFullPath()) )
 
         
-        return (0, namemargin + portmargin + selidx*portpitch + instanceportheight//2)
+        return (0, LogicSymbol.namemargin + LogicSymbol.portmargin + selidx*LogicSymbol.portpitch + LogicSymbol.instanceportheight//2)
 
     def getHeight(self):
-        return namemargin + 2*portmargin + max(len(self.obj.inPorts), len(self.obj.outPorts)) * portpitch 
+        return LogicSymbol.namemargin + 2*LogicSymbol.portmargin + max(len(self.obj.inPorts), len(self.obj.outPorts)) * LogicSymbol.portpitch 
     
     def getWidth(self):
-        return 25 * gridsize    
+        return self.instanceWidth    
 
     def getOccupancy(self):
         return {'x':self.x, 'y':self.y, 'w':self.getWidth(), 'h':self.getHeight()}
@@ -73,12 +136,12 @@ class BinaryOperatorSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
         super().__init__(obj, x, y)
         
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
         canvas.drawText(x, y, text=self.obj.name, anchor='w')
-        y = y + namemargin
+        y = y + LogicSymbol.namemargin
 
         canvas.drawText(x+25, y+25+5, self.operator, anchor='c')
         canvas.drawEllipse(x, y, x+50, y+50)
@@ -88,22 +151,22 @@ class BinaryOperatorSymbol(LogicSymbol):
         #canvas.drawArc(x + 20, y, x + 50, y + 40, start=-90, extent=180) #, style=tkinter.ARC, outline='black', fill='white')
 
     def getHeight(self):
-        return 50 + namemargin
+        return 50 + LogicSymbol.namemargin
 
     def getWidth(self):
         return 50 
     
-    def getWireSourcePos(self, wire:Wire):
-        return (self.getWidth(), namemargin + 25)
+    def gePortSourcePos(self, wire:Wire):
+        return (self.getWidth(), LogicSymbol.namemargin + 25)
     
-    def getWireSinkPos(self, wire:Wire):
+    def getPortSinkPos(self, refport):
         selidx = -1
         for idx, port in enumerate(self.obj.inPorts):
-            if (port.wire == wire):
+            if (port.name == refport.name):
                 selidx = idx
                 
         if (selidx == -1):
-            raise Exception('in port not found in {}'.format(self.obj.getFullPath()) )
+            raise Exception('in port {} not found in {}'.format(self.obj.getFullPath()) )
 
         x = int(math.cos(math.pi/4)*25)
         y = int(math.sin(math.pi/4)*25)
@@ -111,7 +174,7 @@ class BinaryOperatorSymbol(LogicSymbol):
         if (selidx == 0):
             y = -y
 
-        return (25-x, namemargin + 25 + y)
+        return (25-x, LogicSymbol.namemargin + 25 + y)
 
 
 class AddSymbol(BinaryOperatorSymbol):
@@ -140,12 +203,12 @@ class AndSymbol(LogicSymbol):
 
         self.h = 20 * self.nins
 
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
         canvas.drawText(x, y, text=self.obj.name, anchor='w')
-        y = y + namemargin
+        y = y + LogicSymbol.namemargin
 
         # the and box would be x[0:50] y[0:30]
         canvas.drawLine(x, y, x + 25, y)
@@ -154,37 +217,39 @@ class AndSymbol(LogicSymbol):
         canvas.drawArc(x , y+2, x + 50, y + self.h, start=-90, stop=90) #, style=tkinter.ARC, outline='black', fill='white')
 
     def getHeight(self):
-        return namemargin + self.h
+        return LogicSymbol.namemargin + self.h
 
     def getWidth(self):
         return 50
     
-    def getWireSourcePos(self, wire:Wire):
-        return (self.getWidth(), namemargin + self.h//2)
+    def getPortSourcePos(self, refport):
+        return (self.getWidth(), LogicSymbol.namemargin + self.h//2)
     
-    def getWireSinkPos(self, wire:Wire):
+    def getPortSinkPos(self, refport):
         selidx = -1
+        inlist = []
         for idx, port in enumerate(self.obj.inPorts):
-            if (port.wire == wire):
+            inlist.append(port.name)
+            if (port.name == refport.name):
                 selidx = idx
                 
         if (selidx == -1):
-            raise Exception('in port not found in {}'.format(self.obj.getFullPath()) )
+            raise Exception('in port {} not found in {}. Object in ports: {}'.format(refport.name, self.obj.getFullPath(), inlist) )
 
         y = 10 + selidx * 20
 
-        return (5, namemargin + y)
+        return (5, LogicSymbol.namemargin + y)
 
 class NotSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
         super().__init__(obj, x, y)
 
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
         canvas.drawText(x, y, text=self.obj.name, anchor='w')
-        y = y + namemargin 
+        y = y + LogicSymbol.namemargin 
 
         y = y + 10
 
@@ -197,23 +262,23 @@ class NotSymbol(LogicSymbol):
     def getWidth(self):
         return 40
     
-    def getWireSourcePos(self, wire:Wire):
-        return (self.getWidth(), namemargin + 25)
+    def getPortSourcePos(self, port):
+        return (self.getWidth(), LogicSymbol.namemargin + 25)
     
-    def getWireSinkPos(self, wire:Wire):
-        return (0, namemargin + 25)
+    def getPortSinkPos(self, port):
+        return (0, LogicSymbol.namemargin + 25)
     
     
 class BufSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
         super().__init__(obj, x, y)
 
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
         canvas.drawText(x, y, text=self.obj.name, anchor='w')
-        y = y + namemargin 
+        y = y + LogicSymbol.namemargin 
 
         canvas.drawPolygon([x, x + 20, x, x], [y, y + 10, y + 20, y])
         
@@ -223,22 +288,22 @@ class BufSymbol(LogicSymbol):
     def getWidth(self):
         return 20
 
-    def getWireSourcePos(self, wire:Wire):
-        return (self.getWidth(), namemargin + 10)
+    def getWireSourcePos(self, port):
+        return (self.getWidth(), LogicSymbol.namemargin + 10)
     
-    def getWireSinkPos(self, wire:Wire):
-        return (0, namemargin + 10)
+    def getWireSinkPos(self, port):
+        return (0, LogicSymbol.namemargin + 10)
     
 class BitSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
         super().__init__(obj, x, y)
 
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
         canvas.drawText(x, y, text='[{}]'.format(self.obj.bit), anchor='w')
-        y = y + namemargin 
+        y = y + LogicSymbol.namemargin 
 
         canvas.drawLine(x+10,y, x, y+20)
         #canvas.drawPolygon([x, x + 20, x, x], [y, y + 10, y + 20, y])
@@ -249,22 +314,22 @@ class BitSymbol(LogicSymbol):
     def getWidth(self):
         return 20
 
-    def getWireSourcePos(self, wire:Wire):
-        return (10, namemargin + 10)
+    def getPortSourcePos(self, port):
+        return (10, LogicSymbol.namemargin + 10)
     
-    def getWireSinkPos(self, wire:Wire):
-        return (0, namemargin + 10)
+    def getPortSinkPos(self, port):
+        return (0, LogicSymbol.namemargin + 10)
     
 class RangeSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
         super().__init__(obj, x, y)
 
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
         canvas.drawText(x, y, text='[{}:{}]'.format(self.obj.high, self.obj.low), anchor='w')
-        y = y + namemargin 
+        y = y + LogicSymbol.namemargin 
 
         canvas.drawLine(x+10,y, x, y+20)
         #canvas.drawPolygon([x, x + 20, x, x], [y, y + 10, y + 20, y])
@@ -275,23 +340,23 @@ class RangeSymbol(LogicSymbol):
     def getWidth(self):
         return 20
 
-    def getWireSourcePos(self, wire:Wire):
-        return (10, namemargin + 10)
+    def getPortSourcePos(self, port):
+        return (10, LogicSymbol.namemargin + 10)
     
-    def getWireSinkPos(self, wire:Wire):
-        return (0, namemargin + 10)
+    def getPortSinkPos(self, port):
+        return (0, LogicSymbol.namemargin + 10)
     
 class NorSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
         super().__init__(obj, x, y)
         self.h = 40
 
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
         canvas.drawText(x, y, text=self.obj.name, anchor='w')
-        y = y + namemargin
+        y = y + LogicSymbol.namemargin
 
         # the and box would be x[0:50] y[0:30]
         canvas.drawLine(x, y, x + 20, y)
@@ -305,18 +370,18 @@ class NorSymbol(LogicSymbol):
         canvas.drawEllipse(x + 55, y + 15, x + 65, y + 25)
 
     def getHeight(self):
-        return namemargin + self.h
+        return LogicSymbol.namemargin + self.h
 
     def getWidth(self):
         return 65
     
-    def getWireSourcePos(self, wire:Wire):
-        return (self.getWidth(), namemargin + self.h//2)
+    def getPortSourcePos(self, port):
+        return (self.getWidth(), LogicSymbol.namemargin + self.h//2)
     
-    def getWireSinkPos(self, wire:Wire):
+    def getPortSinkPos(self, refport):
         selidx = -1
         for idx, port in enumerate(self.obj.inPorts):
-            if (port.wire == wire):
+            if (port.name == refport.name):
                 selidx = idx
                 
         if (selidx == -1):
@@ -327,7 +392,7 @@ class NorSymbol(LogicSymbol):
         else:
             y = self.h//2 + 10 
 
-        return (5, namemargin + y)
+        return (5, LogicSymbol.namemargin + y)
     
 class OrSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
@@ -340,12 +405,12 @@ class OrSymbol(LogicSymbol):
 
         self.h = 20 * self.nins
         
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
         canvas.drawText(x, y, text=self.obj.name, anchor='w')
-        y = y + namemargin
+        y = y + LogicSymbol.namemargin
 
         # the and box would be x[0:50] y[0:30]
         canvas.drawLine(x, y, x + 20, y)
@@ -359,18 +424,18 @@ class OrSymbol(LogicSymbol):
         canvas.drawSpline([x+20, x+30, x+42, x+50], [y+self.h,  y+self.h-2, y+self.h-self.h//4, y+self.h//2])
 
     def getHeight(self):
-        return namemargin + self.h
+        return LogicSymbol.namemargin + self.h
 
     def getWidth(self):
         return 50
     
-    def getWireSourcePos(self, wire:Wire):
-        return (self.getWidth(), namemargin + self.h//2)
+    def getPortSourcePos(self, port):
+        return (self.getWidth(), LogicSymbol.namemargin + self.h//2)
     
-    def getWireSinkPos(self, wire:Wire):
+    def getPortSinkPos(self, refport):
         selidx = -1
         for idx, port in enumerate(self.obj.inPorts):
-            if (port.wire == wire):
+            if (port.name == refport.name):
                 selidx = idx
                 
         if (selidx == -1):
@@ -378,19 +443,19 @@ class OrSymbol(LogicSymbol):
 
         y = 10 + selidx * 20
 
-        return (5, namemargin + y)
+        return (5, LogicSymbol.namemargin + y)
     
 class XorSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
         super().__init__(obj, x, y)
         self.h = 40
 
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
         canvas.drawText(x, y, text=self.obj.name, anchor='w')
-        y = y + namemargin
+        y = y + LogicSymbol.namemargin
 
         # the and box would be x[0:50] y[0:30]
         canvas.drawLine(x+10, y, x + 20+10, y)
@@ -406,18 +471,18 @@ class XorSymbol(LogicSymbol):
 
 
     def getHeight(self):
-        return namemargin + self.h
+        return LogicSymbol.namemargin + self.h
 
     def getWidth(self):
         return 50+10
     
-    def getWireSourcePos(self, wire:Wire):
-        return (self.getWidth(), namemargin + self.h//2)
+    def getPortSourcePos(self, port):
+        return (self.getWidth(), LogicSymbol.namemargin + self.h//2)
     
-    def getWireSinkPos(self, wire:Wire):
+    def getPortSinkPos(self, refport):
         selidx = -1
         for idx, port in enumerate(self.obj.inPorts):
-            if (port.wire == wire):
+            if (port.name == refport.name):
                 selidx = idx
                 
         if (selidx == -1):
@@ -428,17 +493,17 @@ class XorSymbol(LogicSymbol):
         else:
             y = self.h//2 + 10 
 
-        return (5, namemargin + y)
+        return (5, LogicSymbol.namemargin + y)
     
 class InPortSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
         super().__init__(obj, x, y)
         
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x 
         y = self.y 
         canvas.drawText(x, y, text=self.obj.name , anchor='w')
-        y = y+namemargin 
+        y = y+LogicSymbol.namemargin 
         
         canvas.drawPolygon([x, x+10, x+15, x+10,x,x], [y, y, y+5, y+10, y+10, y])
 
@@ -448,18 +513,18 @@ class InPortSymbol(LogicSymbol):
     def getHeight(self):
         return 20;
     
-    def getWireSourcePos(self, wire:Wire):
-        return (self.getWidth(), namemargin +5)
+    def getPortSourcePos(self, port):
+        return (self.getWidth(), LogicSymbol.namemargin +5)
     
 class OutPortSymbol(LogicSymbol):
     def __init__(self, obj, x, y):
         super().__init__(obj, x, y)
         
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x 
         y = self.y 
         canvas.drawText(x, y, text=self.obj.name , anchor='w')
-        y = y+namemargin 
+        y = y+LogicSymbol.namemargin 
         
         canvas.drawPolygon([x, x+10, x+15, x+10, x, x], [y, y, y+5, y+10, y+10, y])
 
@@ -469,35 +534,61 @@ class OutPortSymbol(LogicSymbol):
     def getHeight(self):
         return 20;
     
-    def getWireSinkPos(self, wire:Wire):
-        return (0, namemargin + 5)
+    def getPortSinkPos(self, port):
+        return (0, LogicSymbol.namemargin + 5)
+
+class InOutPortSymbol(LogicSymbol):
+    def __init__(self, obj, x, y):
+        super().__init__(obj, x, y)
+        
+    def draw(self, canvas, debug=False):
+        x = self.x 
+        y = self.y 
+        canvas.drawText(x, y, text=self.obj.name , anchor='w')
+        y = y+LogicSymbol.namemargin 
+        
+        canvas.drawPolygon([x, x+5, x+10, x+15, x+10, x+5, x], [y+5, y, y, y+5, y+10, y+10, y+5])
+
+    def getWidth(self):
+        return 20;
+    
+    def getHeight(self):
+        return 20;
+    
+    def getPortSinkPos(self, port):
+        return (0, LogicSymbol.namemargin + 5)
+    
+    def getPortSourcePos(self, port):
+        return (self.getWidth(), LogicSymbol.namemargin +5)
+
     
 class InstanceSymbol(LogicSymbol):
     def __init__(self, obj:Logic, x:int, y:int):
         super().__init__(obj, x, y)
     
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x 
         y = self.y 
         
         canvas.drawText(x, y, text=self.obj.name, anchor='w')
-        y = y+namemargin 
+        y = y + LogicSymbol.namemargin 
         
-        canvas.drawRectangle(x, y, x + self.getWidth(), y + self.getHeight() - namemargin)
+        #print('instance symbol width:', self.getWidth())
+        canvas.drawRectangle(x, y, x + self.getWidth(), y + self.getHeight() - LogicSymbol.namemargin)
 
-        ipw = instanceportwidth 
-        iph = instanceportheight
+        ipw = LogicSymbol.instanceportwidth 
+        iph = LogicSymbol.instanceportheight
         iphh = iph//2
-        iptm = instanceporttextmargin
+        iptm = LogicSymbol.instanceporttextmargin
 
-        y = y + portmargin
+        y = y + LogicSymbol.portmargin
         
         for inp in self.obj.inPorts:
             canvas.drawPolygon([x, x+ipw, x+ipw+iphh, x+ipw, x,x], [y, y, y+iphh, y+iph, y+iph,y])
             canvas.drawText(x+ipw+iph, y+iptm, text=inp.name , anchor='w')
-            y = y+portpitch
+            y = y+LogicSymbol.portpitch
             
-        y = self.y + namemargin + portmargin
+        y = self.y + LogicSymbol.namemargin + LogicSymbol.portmargin
         
 
         for inp in self.obj.outPorts:
@@ -505,7 +596,14 @@ class InstanceSymbol(LogicSymbol):
             canvas.drawPolygon([x, x+ipw, x+ipw+iphh, x+ipw, x, x], [y, y, y+iphh, y+iph, y+iph, y])
             x = self.x + self.getWidth() - ipw - iph -iphh
             canvas.drawText(x, y+iptm, text=inp.name , anchor='e')
-            y = y+portpitch
+            y = y+LogicSymbol.portpitch
+
+        for inp in self.obj.inOutPorts:
+            x = self.x + self.getWidth() - ipw - iphh 
+            canvas.drawPolygon([x, x+ipw, x+ipw+iphh, x+ipw, x, x], [y, y, y+iphh, y+iph, y+iph, y])
+            x = self.x + self.getWidth() - ipw - iph -iphh
+            canvas.drawText(x, y+iptm, text=inp.name , anchor='e')
+            y = y+LogicSymbol.portpitch
         
         
 class RegSymbol(InstanceSymbol):
@@ -525,44 +623,44 @@ class ScopeSymbol(InstanceSymbol):
     def getHeight(self):
         return 80
     
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x 
         y = self.y 
         
         canvas.drawText(x, y, text=self.obj.name, anchor='w')
-        y = y+namemargin 
+        y = y+LogicSymbol.namemargin 
 
         canvas.setFillcolor('lightsalmon')
-        canvas.drawRectangle(x, y, x + self.getWidth(), y + self.getHeight() - namemargin, fill=True)
+        canvas.drawRectangle(x, y, x + self.getWidth(), y + self.getHeight() - LogicSymbol.namemargin, fill=True)
 
-        ipw = instanceportwidth 
-        iph = instanceportheight
+        ipw = LogicSymbol.instanceportwidth 
+        iph = LogicSymbol.instanceportheight
         iphh = iph//2
-        iptm = instanceporttextmargin
+        iptm = LogicSymbol.instanceporttextmargin
 
-        y = y + portmargin
+        y = y + LogicSymbol.portmargin
         
         for inp in self.obj.inPorts:
             canvas.drawPolygon([x, x+ipw, x+ipw+iphh, x+ipw, x,x], [y, y, y+iphh, y+iph, y+iph,y])
             #canvas.drawText(x+ipw+iph, y+iptm, text=inp.name , anchor='w')
-            y = y+portpitch
+            y = y+LogicSymbol.portpitch
             
-        y = self.y + namemargin + portmargin
+        y = self.y + LogicSymbol.namemargin + LogicSymbol.portmargin
         
         canvas.setFillcolor('white')
-        canvas.drawRoundRectangle(x+25, y+20, x+self.getWidth()-25, y-20+self.getHeight()-namemargin-20, radius=10, fill=True)
+        canvas.drawRoundRectangle(x+25, y+20, x+self.getWidth()-25, y-20+self.getHeight()-LogicSymbol.namemargin-20, radius=10, fill=True)
         
 class Mux2Symbol(LogicSymbol):
     def __init__(self, obj, x, y):
         super().__init__(obj, x, y)
         self.h = 20*3
 
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
         canvas.drawText(x, y, text=self.obj.name, anchor='w')
-        y = y + namemargin
+        y = y + LogicSymbol.namemargin
 
         canvas.setForecolor('blueviolet')  
         canvas.setLineWidth(1)
@@ -590,18 +688,18 @@ class Mux2Symbol(LogicSymbol):
 
 
     def getHeight(self):
-        return namemargin + self.h
+        return LogicSymbol.namemargin + self.h
 
     def getWidth(self):
         return 20
     
-    def getWireSourcePos(self, wire:Wire):
-        return (self.getWidth(), namemargin + 40)
+    def getPortSourcePos(self, port):
+        return (self.getWidth(), LogicSymbol.namemargin + 40)
     
-    def getWireSinkPos(self, wire:Wire):
+    def getPortSinkPos(self, refport):
         selidx = -1
         for idx, port in enumerate(self.obj.inPorts):
-            if (port.wire == wire):
+            if (port.name == refport.name):
                 selidx = idx
                 
         if (selidx == -1):
@@ -614,12 +712,11 @@ class Mux2Symbol(LogicSymbol):
         else:
             y = 50 
 
-        return (0, namemargin + y)
+        return (0, LogicSymbol.namemargin + y)
 
 class PassthroughSymbol(LogicSymbol):
     def __init__(self):
         super().__init__(None, 0, 0)
-        self.debug = False
         
     def getHeight(self):
         return 30
@@ -627,11 +724,11 @@ class PassthroughSymbol(LogicSymbol):
     def getWidth(self):
         return 30
     
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
-        if (self.debug):       
+        if (debug):       
             canvas.setFillcolor('yellow')  
             canvas.drawRectangle(x, y, x+30, y+30, fill=True)
         else:
@@ -643,16 +740,15 @@ class PassthroughSymbol(LogicSymbol):
             canvas.setForecolor('k')  
             canvas.setLineWidth(2)
         
-    def getWireSinkPos(self, wire:Wire):
+    def getPortSinkPos(self, port):
         return (0, 15);
     
-    def getWireSourcePos(self, wire:Wire):
+    def getPortSourcePos(self, port):
         return (30, 15);
     
 class FeedbackStartSymbol(LogicSymbol):
     def __init__(self):
         super().__init__(None, 0, 0)
-        self.debug = False
         
     def getHeight(self):
         return 30
@@ -660,11 +756,11 @@ class FeedbackStartSymbol(LogicSymbol):
     def getWidth(self):
         return 30
     
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
-        if (self.debug):       
+        if (debug):       
             canvas.setFillcolor('lightgreen')  
             canvas.drawRectangle(x, y, x+30, y+30, fill=True)
         else:            
@@ -677,10 +773,10 @@ class FeedbackStartSymbol(LogicSymbol):
             # canvas.setLineWidth(2)
             pass
         
-    def getWireSinkPos(self, wire:Wire):
+    def getPortSinkPos(self, port):
         return (0, 15);
     
-    def getWireSourcePos(self, wire:Wire):
+    def getPortSourcePos(self, port):
         return (30, 15);
     
 class FeedbackStopSymbol(LogicSymbol):
@@ -694,11 +790,11 @@ class FeedbackStopSymbol(LogicSymbol):
     def getWidth(self):
         return 30
     
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         x = self.x
         y = self.y
 
-        if (self.debug):       
+        if (debug):       
             canvas.setFillcolor('red')  
             canvas.drawRectangle(x, y, x+30, y+30, fill=True)
         else:
@@ -712,15 +808,15 @@ class FeedbackStopSymbol(LogicSymbol):
             # canvas.setLineWidth(2)
             pass
         
-    def getWireSinkPos(self, wire:Wire):
+    def getPortSinkPos(self, port):
         return (0, 15);
     
-    def getWireSourcePos(self, wire:Wire):
+    def getPortSourcePos(self, port):
         return (30, 15);
     
     
 class NetSymbol:
-    def __init__(self, wire:Wire, source:LogicSymbol, sink:LogicSymbol):
+    def __init__(self, wire:Wire, sourcePort, sinkPort, source:LogicSymbol, sink:LogicSymbol):
         """
         A net symbol connects two entities.
 
@@ -739,6 +835,8 @@ class NetSymbol:
 
         """
         self.wire = wire
+        self.sourcePort = sourcePort
+        self.sinkPort = sinkPort
         self.source = source
         self.sink = sink
         self.x = None
@@ -751,19 +849,19 @@ class NetSymbol:
         
     def getStartPoint(self):
         objsource = self.source
-        portsource = objsource.getWireSourcePos(self.wire)
+        portsource = objsource.getPortSourcePos(self.sourcePort)
         return (objsource.x + portsource[0], objsource.y + portsource[1]) 
     
     def getEndPoint(self):
         objsink = self.sink
-        portsink = objsink.getWireSinkPos(self.wire)
+        portsink = objsink.getPortSinkPos(self.sinkPort)
         return (objsink.x + portsink[0], objsink.y + portsink[1]) 
     
     def setPath(self, x, y):
         self.x = x
         self.y = y
         
-    def draw(self, canvas):
+    def draw(self, canvas, debug=False):
         if (self.x == None):
             return
         
